// -*-C++-*-
/*!
 * @file  HiroGateRTCSVC_impl.h
 * @brief Service implementation header of HiroGateRTC.idl
 *
 */

#include "ExtendedDataTypesSkel.h"
#include "BasicDataTypeSkel.h"

#include "HiroGateRTCSkel.h"

#ifndef HIROGATERTCSVC_IMPL_H
#define HIROGATERTCSVC_IMPL_H
 
// -------- HiroGateService -------- //
#include <rtm/CorbaPort.h>
#include "RobotHardwareServiceStub.h"
#include "SequencePlayerServiceStub.h"
#include "RobotHardwareServiceStub.h"
#include "ArmControlServiceStub.h"
#include "GrasperServiceStub.h"
// -------- HiroGateService -------- //


#if defined( _V0514_2 ) || defined( _V0514_5 ) || defined( _V0514_6 )
#include "ikutil.h"
#include "../VSCommon/opencv/cvmath/cvmath.h"
//#include "../VSCommon/system/crtsec/crtsec.h"
#endif


/*!
 * @class HiroGateServiceSVC_impl
 * Example class implementing IDL interface HiroGateRTC::HiroGateService
 */
class HiroGateServiceSVC_impl
 : public virtual POA_HiroGateRTC::HiroGateService,
   public virtual PortableServer::RefCountServantBase
{
 private:
   // Make sure all instances are built on the heap by making the
   // destructor non-public
   //virtual ~HiroGateServiceSVC_impl();

 public:
  /*!
   * @brief standard constructor
   */
   HiroGateServiceSVC_impl();
	 // -------- HiroGateService -------- //
	 HiroGateServiceSVC_impl(
		 RTC::CorbaConsumer<OpenHRP::RobotHardwareService> *rbhw_if,
		 RTC::CorbaConsumer<OpenHRP::SequencePlayerService> *sqply_if,
		 RTC::CorbaConsumer<OpenHRP::ArmControlService> *acsL_if,
		 RTC::CorbaConsumer<OpenHRP::ArmControlService> *acsR_if,
		 RTC::CorbaConsumer<OpenHRP::GrasperService> *gs_if );
	 bool loadPoseDB(const char *file_path);
	 // -------- HiroGateService -------- //

  /*!
   * @brief destructor
   */
   virtual ~HiroGateServiceSVC_impl();

   // attributes and operations
   CORBA::Long getHandPoseL(RTC::Pose3D& pose_l_m_rad);
   CORBA::Long getHandPoseR(RTC::Pose3D& pose_r_m_rad);
	 CORBA::Long getJointAngles(HiroGateRTC::HiroGateService::JointAngles_out angles_rad);
   CORBA::Long setHandPoseL(const RTC::Pose3D& pose_l_m_rad, CORBA::Double rate, CORBA::Boolean wait_for_motion);
   CORBA::Long setHandPoseR(const RTC::Pose3D& pose_r_m_rad, CORBA::Double rate, CORBA::Boolean wait_for_motion);
   CORBA::Long setJointAngles(const HiroGateRTC::HiroGateService::JointAngles& angles_rad, CORBA::Double t, CORBA::Boolean wait_for_motion);
   CORBA::Long setFingerDistanceL(CORBA::Double distance_l_mm, CORBA::Double t, CORBA::Boolean wait_for_motion);
   CORBA::Long setFingerDistanceR(CORBA::Double distance_r_mm, CORBA::Double t, CORBA::Boolean wait_for_motion);
   CORBA::Long setFingerAnglesL(const HiroGateRTC::HiroGateService::FingerAngles& angles_l_rad, CORBA::Double t, CORBA::Boolean wait_for_motion);
   CORBA::Long setFingerAnglesR(const HiroGateRTC::HiroGateService::FingerAngles& angles_r_rad, CORBA::Double t, CORBA::Boolean wait_for_motion);
   CORBA::Boolean isHandMovingL();
   CORBA::Boolean isHandMovingR();
   CORBA::Boolean isJointMoving();
   CORBA::Boolean isFingerMovingL();
   CORBA::Boolean isFingerMovingR();
   CORBA::Long setHandPoseLR(const RTC::Pose3D& pose_l_m_rad, const RTC::Pose3D& pose_r_m_rad, CORBA::Double rate, CORBA::Boolean wait_for_motion);
   CORBA::Long setFingerDistanceLR(CORBA::Double distance_l_mm, CORBA::Double distance_r_mm, CORBA::Double t, CORBA::Boolean wait_for_motion);
   CORBA::Long engageProtectiveStop();
	 CORBA::Long disengageProtectiveStop();
	 CORBA::Long turnServo(CORBA::Boolean body_head_on, CORBA::Boolean armL_on, CORBA::Boolean armR_on, CORBA::Boolean handL_on, CORBA::Boolean handR_on);
	 CORBA::Long setPose(CORBA::Long pose_id, CORBA::Double t, CORBA::Boolean wait_for_motion);
	 //CORBA::Long outXperimental(HiroGateRTC::HiroGateService::Xperimental_out xperimental);
	 CORBA::Boolean getIKSolutionL(const RTC::Pose3D& poseR_l_m_rad, CORBA::Double torso_angle_rad, HiroGateRTC::HiroGateService::JointAngles_out angles_l_rad);
	 CORBA::Boolean getIKSolutionR(const RTC::Pose3D& poseR_r_m_rad, CORBA::Double torso_angle_rad, HiroGateRTC::HiroGateService::JointAngles_out angles_r_rad);
	 CORBA::Boolean isIKSolutionFoundL(const RTC::Pose3D& poseR_l_m_rad, CORBA::Double torso_angle_rad);
	 CORBA::Boolean isIKSolutionFoundR(const RTC::Pose3D& poseR_r_m_rad, CORBA::Double torso_angle_rad);

private:
	// -------- HiroGateService -------- //
	struct service_interface_pointers
	{
		RTC::CorbaConsumer<OpenHRP::RobotHardwareService> *rbhw_if;
		RTC::CorbaConsumer<OpenHRP::SequencePlayerService> *sqply_if;
		RTC::CorbaConsumer<OpenHRP::ArmControlService> *acsL_if;
		RTC::CorbaConsumer<OpenHRP::ArmControlService> *acsR_if;
		RTC::CorbaConsumer<OpenHRP::GrasperService> *gs_if;

	public:
		service_interface_pointers()
			:	rbhw_if( NULL ), sqply_if( NULL ),
			acsL_if( NULL ), acsR_if( NULL ), gs_if( NULL )
		{}

		service_interface_pointers(
			RTC::CorbaConsumer<OpenHRP::RobotHardwareService> *_rbhw_if,
			RTC::CorbaConsumer<OpenHRP::SequencePlayerService> *_sqply_if,
			RTC::CorbaConsumer<OpenHRP::ArmControlService> *_acsL_if,
			RTC::CorbaConsumer<OpenHRP::ArmControlService> *_acsR_if,
			RTC::CorbaConsumer<OpenHRP::GrasperService> *_gs_if )
			:	rbhw_if( _rbhw_if ), sqply_if( _sqply_if ),
			acsL_if( _acsL_if ), acsR_if( _acsR_if ), gs_if( _gs_if )
		{}
	} svc_ptrs;

	HiroGateRTC::HiroGateService::PoseDB pdb;

#	if defined( _V0514_2 ) || defined( _V0514_5 ) || defined( _V0514_6 )

	class IKBase
	{
	public:
		static const double RATE2SECOND;
		static const int N_JOINTS = 6;
	public:
		std::vector< ikfast::IKSolution > solutions;			// Solutions
		ikfast::IKReal solution[N_JOINTS];								// Solution
		std::vector< int > valid_indices;									// Valid indices
		HiroGateRTC::HiroGateService::JointAngles angles;	// JointAngles
		//sys::CriticalSection cs;

	public:
		IKBase();
		virtual ~IKBase();

	public:
		void clear();
		
		void convertSolutionToJointAngles( int part_idx );
		virtual void convertSolutionToJointAngles() = 0;
		
		long solve(
			const RTC::Pose3D& pose_m_rad, 
			bool (*ik_solver)( const ikfast::IKReal*, const ikfast::IKReal*, const ikfast::IKReal*, std::vector< ikfast::IKSolution >& ),
			bool (*ik_filter)( const std::vector< ikfast::IKSolution >&, std::vector< int > & ) );
		virtual long solve( const RTC::Pose3D& pose_m_rad ) = 0;
		virtual long returnIKFailed() = 0;
	private:
		void convertPose3DToRt( const RTC::Pose3D& pose_m_rad,
			opencv::cvmath::Vector< CV_64F > &t,
			opencv::cvmath::Matrix< CV_64F > &R );

		//void lock() { cs.enter(); }
		//void unlock() { cs.exit(); }

	};

	class IKL : public IKBase
	{
	public:
		IKL();
		~IKL();
		void convertSolutionToJointAngles();
		long solve( const RTC::Pose3D& pose_m_rad );
		long returnIKFailed() { return -1; }
	};
	class IKR : public IKBase
	{
	public:
		IKR();
		~IKR();
		void convertSolutionToJointAngles();
		long solve( const RTC::Pose3D& pose_m_rad );
		long returnIKFailed() { return -2; }
	};

#	endif //---------------------------------------------------------------------

	// -------- HiroGateService -------- //

	// L/R commons
	long getHandPose(
		RTC::CorbaConsumer<OpenHRP::ArmControlService> *acs_if,
		RTC::Pose3D& pose_m_rad );

# if defined( _V0514_5 ) //-----------------------------------------

	// setHandPoseL/R()̋ʕ(ikfastO`FbN)
	long setHandPose(
		RTC::CorbaConsumer<OpenHRP::ArmControlService> *acs_if,	IKBase *ik, 
		const RTC::Pose3D& pose_m_rad, double rate, bool wait_for_motion );

#	endif

#	if defined( _V0514_2 ) //---------------------------------------

	// setHandPoseL/R()̋ʕ(ikfast + setJointAngles())
	long setHandPose(
		IKBase *ik, 
		const RTC::Pose3D& pose_m_rad, double rate, bool wait_for_motion );

#	else //----------------------------------------------------------------------

	// setHandPoseL/R()̋ʕ(setTargetAngular())
	long setHandPose(
		RTC::CorbaConsumer<OpenHRP::ArmControlService> *acs_if,
		const RTC::Pose3D& pose_m_rad, double rate, bool wait_for_motion );

#	endif //---------------------------------------------------------------------


#	if defined( _V0514_2 ) || defined( _V0514_5 ) || defined( _V0514_6 )

	// isIKSolutionFoundL/R()̋ʕ
	bool isIKSolutionFound(	IKBase *ik, const RTC::Pose3D& poseR_m_rad, double torso_angle_rad );

	// getIKSolutionL/R()̋ʕ
	bool getIKSolution(
		IKBase *ik, const RTC::Pose3D& pose_m_rad, CORBA::Double torso_angle_rad, 
		HiroGateRTC::HiroGateService::JointAngles_out &angles_rad );

#	endif //---------------------------------------------------------------------

	long setFingerDistance(
		const char *hand_name, double distance_mm, double t, bool wait_for_motion );
	long setFingerAngles(
		const char *hand_name,
		const HiroGateRTC::HiroGateService::FingerAngles& angles_rad,
		CORBA::Double t, bool wait_for_motion );

	long servo( const char *part_name, bool on, long rc_by_error );

private:
	// -------- Type converters -------- //
	// setJointAnglesp̃p[^ϊ
	void convertJointAngles2KeyPose(
		const HiroGateRTC::HiroGateService::JointAngles &angles_rad,
		OpenHRP::KeyPose &pose );

	long convertFingerDistanceToFingerAngles(
		double distance_mm, OpenHRP::GrasperService::DblSequence &angles_rad );

	// -------- Type converters -------- //
private: 
	// -------- Utilities -------- //
	size_t index( const char *str, char c ) {
		int i;
		for( i = 0; *(str + i) != c; ++ i );
		return i;
	}

	void storeJointPartAngles( int pose_id,
		int part_idx, int n_joints, const double *angles )
	{
		pdb[pose_id][part_idx].length(n_joints);
		for( int i = 0; i < n_joints; ++ i )
			pdb[pose_id][part_idx][i] = angles[i];
	}
	// -------- Utilities -------- //

};



#endif // HIROGATERTCSVC_IMPL_H


