#include "HiroGateRTCSVC_impl.h"
#include <cmath>
#include <fstream>
#include <string>
#include "../VSCommon/system/misc/misc.h"


#if defined( _V0514_2 ) || defined( _V0514_5 ) || defined( _V0514_6 )
using namespace ikfast;
using namespace opencv;
using namespace std;
#endif


/*=========================================================
=========================================================*/
CORBA::Long HiroGateServiceSVC_impl::setFingerAnglesL(
	const HiroGateRTC::HiroGateService::FingerAngles& angles_l_rad, 
	CORBA::Double t, CORBA::Boolean wait_for_motion)
{
  return setFingerAngles( "LHAND", angles_l_rad, t, wait_for_motion );
}

CORBA::Long HiroGateServiceSVC_impl::setFingerAnglesR(
	const HiroGateRTC::HiroGateService::FingerAngles& angles_r_rad, 
	CORBA::Double t, CORBA::Boolean wait_for_motion)
{
	return setFingerAngles( "RHAND", angles_r_rad, t, wait_for_motion );
}


/*=========================================================
	setFingerAnglesL/R()̋ʕ
=========================================================*/
long HiroGateServiceSVC_impl::setFingerAngles(
		const char *hand_name, 
		const HiroGateRTC::HiroGateService::FingerAngles& angles_rad,
		CORBA::Double t, bool wait_for_motion )
{
	if( CORBA::is_nil( svc_ptrs.gs_if->getObject() ) )
		return 0x80000000;

	OpenHRP::GrasperService::DblSequence gs_angles;
	gs_angles.length(4);
	gs_angles[0] = angles_rad[0];		gs_angles[1] = angles_rad[1];
	gs_angles[2] = angles_rad[2];		gs_angles[3] = angles_rad[3];

	CORBA::Double ttm;
	(*svc_ptrs.gs_if)->setJointAngles( hand_name, gs_angles, t, ttm );

	if( wait_for_motion )
		(*svc_ptrs.gs_if)->isEmpty( 0x1F, true );

	return 0;
}


/*=========================================================
=========================================================*/
long HiroGateServiceSVC_impl::convertFingerDistanceToFingerAngles(
	double distance_mm, OpenHRP::GrasperService::DblSequence &angles_rad)
{
	/* -pxϊ
	def anglesFromDistance(gripDist):
    
    safetyMargin = 3

    l1 = 33
    l2 = 41.9
    l3 = 30
    l4 = l2 - safetyMargin*2
    l5 = 19

    #if gripDist < 0.0 or gripDist > (l1+l4)*2:
    if gripDist < 0.0 or gripDist > (l2+l5 - safetyMargin)*2:
        return None
    
    xPos   = gripDist/2.0 + safetyMargin
    #print 'xPos =', xPos
    a2Pos  = xPos - l5
    #print 'a2Pos =', a2Pos
    a1radH = math.acos(a2Pos/l2)
    #print 'a1radH = ', a1radH
    a1rad  = math.pi/2.0 - a1radH
    a2rad  = -a1rad
    dEnd   = l2 * math.cos(a1rad) + l3

    return a1rad, a2rad, -a1rad, -a2rad, dEnd
	*/
	const double
		safetyMargin = 3,
		l1 = 33,
		l2 = 41.9,
		l3 = 30,
		l4 = l2 - safetyMargin*2,
		l5 = 19;

	// ~bg`FbN
	if( distance_mm < 0.0 || distance_mm > (l2+l5 - safetyMargin)*2 )
		return -1;

	const double xPos   = distance_mm / 2.0 + safetyMargin;
	const double a2Pos  = xPos - l5;
	const double a1radH = acos(a2Pos/l2);
	const double a1rad  = M_PI / 2.0 - a1radH;
	const double a2rad  = -a1rad;
	// dEnd   = l2 * math.cos(a1rad) + l3

	angles_rad[0] = +a1rad;		angles_rad[1] = +a2rad;
	angles_rad[2] = -a1rad;		angles_rad[3] = -a2rad;

	return 0;
}

