// -*- C++ -*-
/*!
 * @file  HiroPickerAndPlacer.cpp
 * @brief ${rtcParam.description}
 * @date $Date$
 *
 * $Id$
 */

#include "HiroPickerAndPlacer.h"


using std::cout;
using std::cerr;
using std::endl;


// Module specification
// <rtc-template block="module_spec">
static const char* hiropickerandplacer_spec[] =
  {
    "implementation_id", "HiroPickerAndPlacer",
    "type_name",         "HiroPickerAndPlacer",
    "description",       "${rtcParam.description}",
    "version",           "1.0.0",
    "vendor",            "AISL.TUT",
    "category",          "StateMachineGateway",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "0",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
		"conf.default.xWorldR_m","+0.350",
		"conf.default.yWorldR_m","-0.096",
		"conf.default.zWorldR_m","+0.032",
		"conf.default.yWorldR_deg", "0.0",
    
		"conf.default.xPalletPlacedB_m", "+0.470",
		"conf.default.yPalletPlacedB_m", "+0.000",
		"conf.default.zPalletPlacedB_m", "+0.015",

    "conf.default.rPalletPlacedB_deg", "0.0",
		"conf.default.pPalletPlacedB_deg", "-90.0",
		"conf.default.yPalletPlacedB_deg", "0.0",

    "conf.default.directionPalletPlacedR_deg", "100.0",
		
		"conf.default.liftDistancePalletR_m", "0.15",

    "conf.default.xPalletToBePlacedB_m", "+0.470",
		"conf.default.yPalletToBePlacedB_m", "+0.000",
		"conf.default.zPalletToBePlacedB_m", "+0.032",

		"conf.default.directionPalletToBePlacedR_deg", "0.0",

		"conf.default.holdDistance_m",     "0.001",
		"conf.default.releaseDistance_m",  "0.002",
		"conf.default.fingerBacklash_deg", "8.0",

    "conf.default.xOPEOffsetR_m", "0.0",
		"conf.default.yOPEOffsetR_m", "0.0",

		"conf.default.speedBase", "5.0",

		"conf.default.serialNo", "3",
    // Widget
    "conf.__widget__.xWorldR_m", "text",
    "conf.__widget__.yWorldR_m", "text",
    "conf.__widget__.zWorldR_m", "text",
		"conf.__widget__.yWorldR_deg", "text",

    "conf.__widget__.xPalletPlacedB_m", "text",
    "conf.__widget__.yPalletPlacedB_m", "text",
    "conf.__widget__.zPalletPlacedB_m", "text",

    "conf.__widget__.rPalletPlacedB_deg", "text",
    "conf.__widget__.pPalletPlacedB_deg", "text",
    "conf.__widget__.yPalletPlacedB_deg", "text",

    "conf.__widget__.directionPalletPlacedR_deg", "text",
    
		"conf.__widget__.liftDistancePalletR_m", "text",

    "conf.__widget__.xPalletToBePlacedB_m", "text",
    "conf.__widget__.yPalletToBePlacedB_m", "text",
    "conf.__widget__.zPalletToBePlacedB_m", "text",

    "conf.__widget__.directionPalletToBePlacedR_deg", "text",

		"conf.__widget__.holdDistance_m",     "text",
		"conf.__widget__.releaseDistance_m",  "text",
		"conf.__widget__.fingerBacklash_deg", "text",

    "conf.__widget__.xOPEOffsetR_m", "text",
    "conf.__widget__.yOPEOffsetR_m", "text",

    "conf.__widget__.speedBase", "text",

    "conf.__widget__.serialNo", "text",
    // Constraints
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
HiroPickerAndPlacer::HiroPickerAndPlacer(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_inputIn("input", m_input),
    m_outputOut("output", m_output),
    m_HiroGateServicePort("HiroGateService"),
		m_ObjectPoseEstimationGateServicePort("ObjectPoseEstimationGateService"),
    m_HiroNXPort("HiroNX"),
    m_HIROPort("HIRO")

    // </rtc-template>
{
	cout << ">> HiroPickerAndPlacer::HiroPickerAndPlacer()" << endl;
	cout << "<< HiroPickerAndPlacer::HiroPickerAndPlacer()" << endl;
}

/*!
 * @brief destructor
 */
HiroPickerAndPlacer::~HiroPickerAndPlacer()
{
	cout << ">> HiroPickerAndPlacer::~HiroPickerAndPlacer()" << endl;
	deactivate();
	cout << "<< HiroPickerAndPlacer::~HiroPickerAndPlacer()" << endl;
}



RTC::ReturnCode_t HiroPickerAndPlacer::onInitialize()
{
	cerr << ">> HiroPnP::onInitialize()" << endl;

  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  addInPort("input", m_inputIn);
  
  // Set OutPort buffer
  addOutPort("output", m_outputOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  m_HiroGateServicePort.registerConsumer("HiroGateSvcIF", "HiroGateRTC::HiroGateService", hg);
	m_ObjectPoseEstimationGateServicePort.registerConsumer("ObjectPoseEstimationGateSvcIF", "ObjectPoseEstimationGateRTC::ObjectPoseEstimationGateService", ope);
  m_HiroNXPort.registerConsumer("manipulator", "HiroNX", manipu);
  m_HIROPort.registerConsumer("common", "CommonCommands", common);
  m_HIROPort.registerConsumer("motion", "MotionCommands", motion);
  
  // Set CORBA Service Ports
  addPort(m_ObjectPoseEstimationGateServicePort);
  addPort(m_HiroGateServicePort);
  addPort(m_HiroNXPort);
  addPort(m_HIROPort);
  
  // </rtc-template>

  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
#	if 0
  bindParameter("xWorldR_m", m_xWorldR_m, "0.285");
  bindParameter("yWorldR_m", m_yWorldR_m, "-0.122");
  bindParameter("zWorldR_m", m_zWorldR_m, "-0.087");
  bindParameter("xPalletPlacedB_m", m_xPalletPlacedB_m, "0.425");
  bindParameter("yPalletPlacedB_m", m_yPalletPlacedB_m, "0.0");
  bindParameter("zPalletPlacedB_m", m_zPalletPlacedB_m, "-0.087");
  bindParameter("rPalletPlacedB_deg", m_rPalletPlacedB_deg, "0.0");
  bindParameter("pPalletPlacedB_deg", m_pPalletPlacedB_deg, "-90.0");
  bindParameter("yPalletPlacedB_deg", m_yPalletPlacedB_deg, "0.0");
  bindParameter("directionPalletPlacedR_deg", m_directionPalletPlacedR_deg, "-90.0");
  bindParameter("liftDistancePalletR_m", m_liftDistancePalletR_m, "0.05");
  bindParameter("xPalletToBePlacedB_m", m_xPalletToBePlacedB_m, "0.425");
  bindParameter("yPalletToBePlacedB_m", m_yPalletToBePlacedB_m, "0.000");
  bindParameter("zPalletToBePlacedB_m", m_zPalletToBePlacedB_m, "-0.087");
  bindParameter("directionPalletToBePlacedR_deg", m_directionPalletToBePlacedR_deg, "0.0");
  bindParameter("xOPEOffsetR_m", m_xOPEOffsetR_m, "0.0");
  bindParameter("yOPEOffsetR_m", m_yOPEOffsetR_m, "0.0");
  bindParameter("speedBase", m_speedBase, "15");
  bindParameter("serialNo", m_serialNo, "3");
  bindParameter("yWorldR_deg", m_yWorldR_deg, "0.0");
#	else
	// EW({bgWnR)
	bindParameter("xWorldR_m", conf.worldR_m(0), "+0.350");
	bindParameter("yWorldR_m", conf.worldR_m(1), "-0.096");
	bindParameter("zWorldR_m", conf.worldR_m(2), "+0.032");
	bindParameter("yWorldR_deg", conf.worldR_deg, "0.0");

	// pbguꏊ(̍WnB)
  bindParameter("xPalletPlacedB_m", conf.pallet.xyz_placedB_m(0), "+0.470");
	bindParameter("yPalletPlacedB_m", conf.pallet.xyz_placedB_m(1), "+0.000");
	bindParameter("zPalletPlacedB_m", conf.pallet.xyz_placedB_m(2), "+0.015");

  bindParameter("rPalletPlacedB_deg", conf.pallet.rpy_placedB_deg(0), "0.0");
	bindParameter("pPalletPlacedB_deg", conf.pallet.rpy_placedB_deg(1), "-90.0");
	bindParameter("yPalletPlacedB_deg", conf.pallet.rpy_placedB_deg(2), "0.0");

	// pbgu
  bindParameter("directionPalletPlacedR_deg", conf.pallet.direction_placedR_deg, "100.0");

	// グ
	bindParameter("liftDistancePalletR_m", conf.pallet.lift_distance_R_m, "0.15");

	// pbguꏊ(̍WnB)
	bindParameter("xPalletToBePlacedB_m", conf.pallet.xyz_to_be_placedB_m(0), "+0.470");
	bindParameter("yPalletToBePlacedB_m", conf.pallet.xyz_to_be_placedB_m(1), "+0.000");
	bindParameter("zPalletToBePlacedB_m", conf.pallet.xyz_to_be_placedB_m(2), "+0.032");

	// pbgu
	bindParameter("directionPalletToBePlacedR_deg", conf.pallet.direction_to_be_placedR_deg, "0.0");

	// ̂͂/Ƃ̎̋ߕ/L
	bindParameter("holdDistance_m",    conf.hold_distance_m,    "0.001");
	bindParameter("releaseDistance_m", conf.release_distance_m, "0.002");

	// w摤̊֐߂̗Vъp
	bindParameter("fingerBacklash_deg", conf.finger_backlash_deg, "8.0");

	// ̈ʒup̂
	bindParameter("xOPEOffsetR_m", conf.xy_ope_offsetR_m(0), "0.0");
	bindParameter("yOPEOffsetR_m", conf.xy_ope_offsetR_m(1), "0.0");

	// 쑬x
	bindParameter("speedBase", conf.speed_base, "5.0");

	// VANo
	bindParameter("serialNo", conf.serial_no, "3");
#	endif

	initializePose();            // p̏
	initializeWorldParameter();  // OpenCVEWñ{bgWn\iRtBM[Vj
	initializeObjectModel();     // f`
		// SequencerRTCƂ͈قȂCOPE󂯎f[^ɃIuWFNgIDCʒup
		// ɉCƕ܂ނ悤ɂD̉ɂC{֐͉ȂD
	initializePalletParameter(); // pbg̃p[^iRtBM[Vj
	initializePlacingSlot();     // uꏊ`

  // </rtc-template>

	cerr << "<< HiroPnP::onInitialize()" << endl;
  return RTC::RTC_OK;
}


RTC::ReturnCode_t HiroPickerAndPlacer::onFinalize()
{
	cerr << ">> HiroPnP::onFinalize()" << endl;
	cerr << "<< HiroPnP::onFinalize()" << endl;
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t HiroPickerAndPlacer::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t HiroPickerAndPlacer::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t HiroPickerAndPlacer::onActivated(RTC::UniqueId ec_id)
{
	cerr << ">> HiroPnP::onActivated()" << endl;
	activate();
	cerr << "<< HiroPnP::onActivated()" << endl;
  return RTC::RTC_OK;
}


RTC::ReturnCode_t HiroPickerAndPlacer::onDeactivated(RTC::UniqueId ec_id)
{
	cerr << ">> HiroPnP::onDeactivated()" << endl;
	cerr << "<< HiroPnP::onDeactivated()" << endl;
  return RTC::RTC_OK;
}


RTC::ReturnCode_t HiroPickerAndPlacer::onExecute(RTC::UniqueId ec_id)
{
	process();
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t HiroPickerAndPlacer::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t HiroPickerAndPlacer::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t HiroPickerAndPlacer::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t HiroPickerAndPlacer::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t HiroPickerAndPlacer::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void HiroPickerAndPlacerInit(RTC::Manager* manager)
  {
    coil::Properties profile(hiropickerandplacer_spec);
    manager->registerFactory(profile,
                             RTC::Create<HiroPickerAndPlacer>,
                             RTC::Delete<HiroPickerAndPlacer>);
  }
  
};


