#ifdef HIROPNP_USE_DOUBLE_ARM_COMMON_IF
#include "../HiroNXGate/nxutil.h"
#endif
#include "HiroPickerAndPlacer.h"
#include "../VSCommon/system/misc/misc.h"
#include <cstdlib>


using namespace pnp;
using namespace std;
using namespace opencv;


typedef HiroGateRTC::HiroGateService::JointAngles JointAngles;
typedef HiroGateRTC::HiroGateService::FingerAngles FingerAngles;


bool operation_t::pose_initialized = false;
JointAngles operation_t::pose_all_initial;
JointAngles operation_t::pose_torso_only;
JointAngles operation_t::pose_arm_initial;
FingerAngles operation_t::fL_pallet_open;
FingerAngles operation_t::fR_pallet_open;
FingerAngles operation_t::f_close;//f_3cm_close;
//cvmath::Matrix< CV_64F > operation_t::R_WtoR(3,3);

sys::Thread operation_t::thread;
//sys::CriticalSection operation_t::cs;
bool operation_t::in_obj_manip = false, operation_t::in_tray_manip = false;
bool operation_t::kill_switch = false;
bool operation_t::is_hiro_connected = false;

std::string operation_t::input, operation_t::output;


/**********************************************************
	Hiro̎p̏
**********************************************************/
bool HiroPickerAndPlacer::initializePose()
{
	printf( "[Initialize] initializing robot pose..." ); fflush( stdout );

	if( operation_t::pose_initialized ) {
		printf( "already initialized\n" );
		return false;
	}

	JointAngles &pose_all_initial = operation_t::pose_all_initial;
	JointAngles &pose_torso_only = operation_t::pose_torso_only;
	JointAngles &pose_arm_initial = operation_t::pose_arm_initial;
	FingerAngles &fL_pallet_open = operation_t::fL_pallet_open;
	FingerAngles &fR_pallet_open = operation_t::fR_pallet_open;
	//FingerAngles &f_3cm_close = operation_t::f_3cm_close;
	FingerAngles &f_close = operation_t::f_close;

	/* p */
	hgutil::allocateJointAngles( pose_all_initial, true, true, true );
	{	// ́E
		pose_all_initial[0][0] = 0.0;
		pose_all_initial[0][1] = 0.0;
		pose_all_initial[0][2] = 0.450;
	}
	{	// Er
		pose_all_initial[1][0] = -0.407;  pose_all_initial[1][1] = -0.126;
		pose_all_initial[1][2] = -2.727;  pose_all_initial[1][3] = +0.125;
		pose_all_initial[1][4] = 0.392;   pose_all_initial[1][5] = +0.056;
	}
	{	// r
		pose_all_initial[2][0] = +0.407;  pose_all_initial[2][1] = -0.126;
		pose_all_initial[2][2] = -2.727;  pose_all_initial[2][3] = -0.125;
		pose_all_initial[2][4] = 0.392;   pose_all_initial[2][5] = -0.056;
	}

	/* ̂̔P */
	hgutil::allocateJointAngles( pose_torso_only, true, false, false );
	{	// ́E
		pose_torso_only[0] = pose_all_initial[0];	// pose_all_initialƓ
	}

	/* ȑp */
	hgutil::allocateJointAngles( pose_arm_initial, false, true, true );
	{	// EE
		pose_arm_initial[1] = pose_all_initial[1];
		pose_arm_initial[2] = pose_all_initial[2];
	}

	/* w */
	fL_pallet_open.length( 4 );
	fR_pallet_open.length( 4 );
	fL_pallet_open[0] = -0.222, fL_pallet_open[1] = +0.208, fL_pallet_open[2] = -0.490, fL_pallet_open[3] = +0.422;
	fR_pallet_open[0] = +0.422, fR_pallet_open[1] = -0.490, fR_pallet_open[2] = +0.208, fR_pallet_open[3] = -0.222;

	/*
	f_3cm_close.length( 4 );
	f_3cm_close[0] = -0.133, f_3cm_close[1] = -0.023, f_3cm_close[2] = 0.096, f_3cm_close[3] = 0.035;

	printf( "[ ref ]  : %.1f, %.1f, %.1f, %.1f\n",  
			cvmath::rad2deg( f_3cm_close[0] ), cvmath::rad2deg( f_3cm_close[1] ),
			cvmath::rad2deg( f_3cm_close[2] ), cvmath::rad2deg( f_3cm_close[3] ) );

	for( int i = 25; i <= 40; ++ i ) {
		hgutil::calculateFingerAnglesFromDistance( i, f_3cm_close );
		printf( "[ %2dmm ]  : %.1f, %.1f, %.1f, %.1f\n", i, 
			cvmath::rad2deg( f_3cm_close[0] ), cvmath::rad2deg( f_3cm_close[1] ),
			cvmath::rad2deg( f_3cm_close[2] ), cvmath::rad2deg( f_3cm_close[3] ) );
	}
	*/

	f_close.length( 4 );

	operation_t::pose_initialized = true;

	printf( "OK\n" );

	return true;
}


/**********************************************************
	̃f̏
**********************************************************/
bool HiroPickerAndPlacer::initializeObjectModel()
{
	return true;
}


/**********************************************************
	EWn̏iRtBM[Vj
**********************************************************/
bool HiroPickerAndPlacer::initializeWorldParameter()
{
	return true;
}


/**********************************************************
	gC̃p[^̏iRtBM[Vj
**********************************************************/
bool HiroPickerAndPlacer::initializePalletParameter()
{
	return true;
}


/**********************************************************
	gC̒uꏊ̏
**********************************************************/
bool HiroPickerAndPlacer::initializePlacingSlot()
{

	printf( "[Initialize] initializing placing slot..." ); fflush( stdout );

	configuration_parameter_t::pallet_param_t &plt = conf.pallet;
	cvutil::Image< IPL_DEPTH_64F > &placing_slotB = plt.placing_slotB;

	plt.placing_slotB.create( 2, 2, 4 );
	plt.placing_slotB.clear();
	
	const int CU = plt.CUBOID, CY = plt.CYLINDER;

	placing_slotB(CU,0, 0) = plt.xyz_placedB_m(0) + plt.xy_slotP_abs_m(0);
	placing_slotB(CU,0, 1) = +plt.xy_slotP_abs_m(1);
	placing_slotB(CU,0, 2) = plt.xyz_placedB_m(2) + plt.bottom_height_m;
	placing_slotB(CU,0, 3) = 0;

	placing_slotB(CU,1, 0) = plt.xyz_placedB_m(0) - plt.xy_slotP_abs_m(0);
	placing_slotB(CU,1, 1) = +plt.xy_slotP_abs_m(1);
	placing_slotB(CU,1, 2) = plt.xyz_placedB_m(2) + plt.bottom_height_m;
	placing_slotB(CU,1, 3) = 0;

	placing_slotB(CY,0, 0) = plt.xyz_placedB_m(0) + plt.xy_slotP_abs_m(0);
	placing_slotB(CY,0, 1) = -plt.xy_slotP_abs_m(1);
	placing_slotB(CY,0, 2) = plt.xyz_placedB_m(2) + plt.bottom_height_m;
	placing_slotB(CY,0, 3) = 0;

	placing_slotB(CY,1, 0) = plt.xyz_placedB_m(0) - plt.xy_slotP_abs_m(0);
	placing_slotB(CY,1, 1) = -plt.xy_slotP_abs_m(1);
	placing_slotB(CY,1, 2) = plt.xyz_placedB_m(2) + plt.bottom_height_m;
	placing_slotB(CY,1, 3) = 0;

	printf( "OK\n" );

	return true;
}


/**********************************************************
	ANeBx[g̏
**********************************************************/
int HiroPickerAndPlacer::activate()
{

	operation_t::in_obj_manip = operation_t::in_tray_manip = false;
	operation_t::kill_switch = false;
	operation_t::is_hiro_connected = false;
	//operation_t::cs.initialize();

	// pbgɂ
	configuration_parameter_t::pallet_param_t &plt = conf.pallet;
	cvutil::Image< IPL_DEPTH_64F > &placing_slotB = plt.placing_slotB;

	for( int v = 0; v < placing_slotB.getHeight(); ++ v )
		for( int u = 0; u < placing_slotB.getWidth(); ++ u )
			placing_slotB(u,v,3) = 0;

	// p
	printf( "[Activate] initial robot pose..." ); fflush( stdout );

#	ifdef HIROPNP_USE_DOUBLE_ARM_COMMON_IF //----------------
#	if 1
	if( !rtcutil::isInOperation( motion ) || !rtcutil::isInOperation( manipu ) ) {
		sys::returnByError< int >( -1,
			"HiroPickerAndPlacer::activate(): "
			"HIROController and HiroNX not found." );
	}
#	else
	if( !rtcutil::isInOperation( hnxg ) ) {
		sys::returnByError< int >( -1,
			"HiroPickerAndPlacer::activate(): "
			"HiroNXGateService not found." );
	}
#endif
	else {
		// HIRO͐ڑς
		operation_t::is_hiro_connected = true;

		MotionCommands::JointPosSeq jpseq;
		MotionCommands::DoubleSeq tmseq;

		jpseq.length( 1 );
		tmseq.length( 1 );

		jpseq[0].length( 24 );
		tmseq[0] = 2;

		// ŵ݂͎̕蓮ŏ
		jpseq[0][15] = jpseq[0][16] = jpseq[0][17] = jpseq[0][18] = 0.0;
		jpseq[0][19] = jpseq[0][20] = jpseq[0][21] = jpseq[0][22] = 0.0;

		nxutil::Hiro2HiroNX( operation_t::pose_all_initial, jpseq[0] );
		nxutil::printJointPos( jpseq[0] );

#		if 1
		manipu->DisengageProtectiveStop();
		motion->movePTPJointAbsSeq( jpseq, tmseq );
#		else
		hnxg->DisengageProtectiveStop();
		hnxg->setJointPosSeq( jpseq, tmseq );
#endif
	}
#	else	//--------------------------------------------------
	if( !rtcutil::isInOperation( hg ) ) {
		sys::returnByError< int >( -1, "HiroPickerAndPlacer::activate(): "
			"HiroGateService not found." );
	}
	else {
		// HIRO͐ڑς
		operation_t::is_hiro_connected = true;

		hg->disengageProtectiveStop();
		hg->setJointAngles( operation_t::pose_all_initial, 4, true );
	}
#	endif //-------------------------------------------------

	printf( "OK\n" );

	return 0;
}


/**********************************************************
	fANeBx[g̏I
**********************************************************/
int HiroPickerAndPlacer::deactivate()
{

	bool &kill_switch = operation_t::kill_switch;
	if( kill_switch ) return 0;

	printf( "[Deactivate] stopping manipulation thread..." ); fflush( stdout );

	// Xbh̒~
	stopThread( &operation_t::in_obj_manip );
	stopThread( &operation_t::in_tray_manip );

	printf( "OK\n" );
	
	//operation_t::cs.finalize();

	return 0;
}


/**********************************************************
	Xbh̒~
**********************************************************/
void HiroPickerAndPlacer::stopThread( bool *in_operation )
{

	if( !*in_operation ) // Xbhɒ~Ăꍇ
		return;

	/* Xbh̏ꍇ */
	// HIRO~ĂXbh~

#	if HIROPNP_USE_DOUBLE_ARM_COMMON_IF
#	if 1
	manipu->EngageProtectiveStop();
#	else
	hnxg->EngageProtectiveStop();
#	endif
#	else
	hg->engageProtectiveStop();
#	endif

	bool &kill_switch = operation_t::kill_switch;
	sys::Thread &thread = operation_t::thread;

	kill_switch = true;
	
	thread.stop();

	*in_operation = false;

}