#include "HiroPickerAndPlacer.h"
#include "../VSCommon/system/misc/misc.h"


using namespace std;
using namespace opencv;


using hgutil::OperationNode;
using hgutil::JointOperation;
using hgutil::FingerAngleOperation;
using hgutil::FingerAngleLROperation;
using hgutil::FingerDistanceOperation;
using hgutil::FingerDistanceLROperation;
using hgutil::HandOperation;
using hgutil::HandLROperation;
using hgutil::SleepOperation;


/**********************************************************
	̂̑̕
**********************************************************/
bool HiroPickerAndPlacer::manipulateSingleObject(
	int model_id,
	const RTC::Size3D &obj_size_m, const RTC::Pose3D &obj_poseR_m_rad )
{

	cout << "[Object Pose] --------------------------------" << endl;
	hgutil::printPose3D( obj_poseR_m_rad );

	int rc;

	cout << "[Object parameters] --------------------------" << endl;
	printf( "ID     = %d\n", model_id );
	printf( "width  = %.1f[mm]\n", 1000*obj_size_m.w );
	printf( "height = %.1f[mm]\n", 1000*obj_size_m.h );
	printf( "yaw    = %.1f[deg]\n", cvmath::rad2deg( obj_poseR_m_rad.orientation.y ) );

	/* 󂫃XbgT */
	RTC::Point3D slot_xyzB_m;
	if( conf.pallet.findEmptyPlacingSlot( model_id, slot_xyzB_m ) ) {
		// XbgȂΒ߂
		return sys::returnByError( false,
			"\a\a\a\aHiroPickerAndPlacer::manipulateSingleObject(): "
			"no empty slot." );
	}

	/* Xbgꍇ */
	vector< OperationNode* > ope_nodes;
	
	/* ̂͂łƂ̎w̒ */
	double finger_depth_mm;
	hgutil::calculateFingerDepthFromDistance( 1000*obj_size_m.w, &finger_depth_mm, true, false );
	finger_depth_mm += (conf.serial_no == 11) ? 59 : 0;	// AISThiro011p
	finger_depth_mm -= 20; // w惉o[ItZbg(w惉o[Ŝ̂ɓp)
	const double finger_depth_m = 0.001*finger_depth_mm;
	
	hgutil::dominant_side_t pck_side;

	rc = planObjectPicking( obj_size_m, obj_poseR_m_rad, slot_xyzB_m, finger_depth_m, ope_nodes, &pck_side );
	if( rc ) {
		return sys::returnByError( false,
			"\a\aHiroPickerAndPlacer::manipulateSingleObject(): "
			"cannot calculate picking parameters." );
	}

	rc = planObjectPlacing( obj_size_m, slot_xyzB_m, finger_depth_m, pck_side, ope_nodes );
	if( rc ) {
		return sys::returnByError( false,
			"\a\aHiroPickerAndPlacer::manipulateSingleObject(): "
			"cannot calculate placing parameters." );
	}

	cout << "[Object picking & placing plan] ----------------" << endl;
	for( vector< OperationNode* >::iterator it = ope_nodes.begin(); it != ope_nodes.end(); ++ it )
		(*it)->print();

	cout << "[Object pick & place] --------------------------" << endl;
#	ifdef HIROPNP_USE_DOUBLE_ARM_COMMON_IF
	rc = actPlan( ope_nodes, &pnp::operation_t::pose_all_initial );
#	else
	rc = actPlan( ope_nodes );
#	endif
	if( rc ) {
		return sys::returnByError( false,
			"\a\aHiroPickerAndPlacer::manipulateSingleObject(): "
			"cannot pick & place object." );
	}

	deletePlan( ope_nodes );

	return true;
}


/**********************************************************
	󂫃XbgT
**********************************************************/
int pnp::configuration_parameter_t::pallet_param_t::
findEmptyPlacingSlot( int model_id, RTC::Point3D &slot_xyzB_m )
{

	// fɑΉXbg
	if( model_id >= placing_slotB.getWidth() )
		return -1;

	for( int slot_idx = 0; slot_idx < placing_slotB.getHeight(); ++ slot_idx )
		if( !placing_slotB( model_id, slot_idx, 3 ) ) {
			// gpɂ
			placing_slotB( model_id, slot_idx, 3 ) = 1;
				// ނCSɒuꍇɎgpς݂ɂׂ?
			
			slot_xyzB_m.x = placing_slotB( model_id, slot_idx, 0 );
			slot_xyzB_m.y = placing_slotB( model_id, slot_idx, 1 );
			slot_xyzB_m.z = placing_slotB( model_id, slot_idx, 2 );
			return 0;
		}
	
	// 󂫃Xbg
	return -1;

}


/**********************************************************
	sbLOv
**********************************************************/
int HiroPickerAndPlacer::planObjectPicking(
	const RTC::Size3D &obj_size_m,
	const RTC::Pose3D &obj_poseR_m_rad, const RTC::Point3D &slot_xyzB_m,
	double finger_depth_m, vector< OperationNode* > &ope_nodes, hgutil::dominant_side_t *pck_side )
{
#	ifdef HIROPNP_USE_DOUBLE_ARM_COMMON_IF
	printf( "Planning / Obj picking (Common I/F)" ), fflush( stdout );
#	else
	printf( "Planning / Obj picking" ), fflush( stdout );
#	endif

	pnp::operation_t pck0;
	RTC::Pose3D R_tmp;

	/* ̂͂ނƂ̓̎p */
	pck0.torso_angle_rad = 0.0; // iЂƂ܂C0j

	/* r */
	pck0.decideDominantSide( obj_poseR_m_rad );
	*pck_side = pck0.side;

	/* ̔c̘r̈ʒupvZ (ʒu̕␳t) */
	// Eʕ␳̗
	// xyzR(0) += conf.xy_ope_offsetR_m(0) + (dominant_side == LEFT) ? (-0.005) : (0.0);
	pck0.R.position.x = obj_poseR_m_rad.position.x + conf.xy_ope_offsetR_m(0);
	pck0.R.position.y = obj_poseR_m_rad.position.y + conf.xy_ope_offsetR_m(1);
	pck0.R.position.z = conf.worldR_m(2) + obj_size_m.h + finger_depth_m;
		// ɊւẮČʂpɁC(Ƒ䍂,̍,w撷)ɌvZ
	pck0.R.orientation.r = obj_poseR_m_rad.orientation.y;
	pck0.R.orientation.p = cvmath::deg2rad( -90 );
	pck0.R.orientation.y = 0;

	hgutil::R2B( pck0.R, pck0.torso_angle_rad, pck0.B );

	/* ẘԊu */
	// ̂u͂ށv̂Ŏw͊Oɉ
	// VуlĂX̊pxƂȂ悤ɕ␳
	HiroGateRTC::HiroGateService::FingerAngles
		&f_close = pnp::operation_t::f_close;
	hgutil::calculateFingerAnglesFromDistance(
		1000*( obj_size_m.w - conf.hold_distance_m ),
		f_close, cvmath::deg2rad( conf.finger_backlash_deg ) );

	putch( '.' ), fflush( stdout );

	/* 1.̂̏ */
	{
		R_tmp = pck0.R;
		R_tmp.position.z += 0.05;

		if( !HandOperation::isIKSolutionFound( 
			hg, pck0.side, R_tmp, pck0.torso_angle_rad ) )
			return sys::returnByError( -1, "IK failed. (1)" );

		ope_nodes.push_back(
			new HandOperation( hg, 
				pck0.side, R_tmp, 1.5*conf.speed_base, true ) );
	}
	putch( '.' ), fflush( stdout );

	/* 2.wJ */
	{
		ope_nodes.push_back(
			new FingerDistanceOperation( hg, 
				pck0.side, 1000*obj_size_m.w + 40, 0.1, true ) );
	}

	putch( '.' ), fflush( stdout );

	/* 3. */
	{
		if( !HandOperation::isIKSolutionFound( hg, 
			pck0.side, pck0.R, pck0.torso_angle_rad ) )
			return sys::returnByError( -1, "IK failed. (3)" );

		ope_nodes.push_back( 
			new HandOperation( hg,
				pck0.side, pck0.R, 0.5*conf.speed_base, true ) );
	}

	putch( '.' ), fflush( stdout );

	/* 4.҂ */
	{
		ope_nodes.push_back( new SleepOperation( 0.2 ) );
	}

	putch( '.' ), fflush( stdout );

	/* 5.w */
	{
		ope_nodes.push_back(
			new FingerAngleOperation( hg,
				// pck0.side, f_3cm_close, 0.1, true ) );
				pck0.side, f_close, 0.1, true ) );
	}

	putch( '.' ), fflush( stdout );

	/* 6.҂ */
	{
		ope_nodes.push_back( new SleepOperation( 0.2 ) );
	}

	putch( '.' ), fflush( stdout );

	/* 7. (̂uĂꏊƃpbg̍قɍ킹Ďグ) */
	{
		// uꏊ̍s
		const double z_plc = slot_xyzB_m.z + ( obj_size_m.h + finger_depth_m ) + conf.pallet.height_m + 0.05;
		const double z_pck = pck0.R.position.z + 0.15;

		R_tmp = pck0.R;
		R_tmp.position.z = ( z_plc > z_pck ? z_plc : z_pck );

		if( !HandOperation::isIKSolutionFound( hg,
			pck0.side, R_tmp, pck0.torso_angle_rad ) )
			return sys::returnByError( -1, "IK failed. (7)" );

		ope_nodes.push_back(
			new HandOperation( hg, 
				pck0.side, R_tmp, conf.speed_base, true ) );
	}

	printf( "[OK]\n" );

	return 0;
}


/**********************************************************
	v[VOv
**********************************************************/
int HiroPickerAndPlacer::planObjectPlacing(
	const RTC::Size3D &obj_size_m, const RTC::Point3D &slot_xyzB_m, 
	double finger_depth_m, hgutil::dominant_side_t picking_side,
	vector< OperationNode* > &ope_nodes )
{
#	ifdef HIROPNP_USE_DOUBLE_ARM_COMMON_IF
	printf( "Planning / Obj placing (Common I/F)" ), fflush( stdout );
#	else
	printf( "Planning / Obj placing" ), fflush( stdout );
#	endif
	

	pnp::operation_t plc0;
	RTC::Pose3D R_tmp;
	HiroGateRTC::HiroGateService::JointAngles torso_angles_rad;
	//hgutil::copyJointAngles( pnp::operation_t::pose_torso_only, torso_angles_rad );
	torso_angles_rad = pnp::operation_t::pose_torso_only;

	/* r */
	plc0.side = picking_side;

	/* gCւ̐ݒu̘r̈ʒupvZ */
	plc0.B.position = slot_xyzB_m;
	plc0.B.position.z += ( obj_size_m.h + finger_depth_m );
		// (gC,̍,w撷)ɌvZ
	plc0.B.orientation.r = cvmath::deg2rad( 0 );
	plc0.B.orientation.p = cvmath::deg2rad( -90 );
	plc0.B.orientation.y = cvmath::deg2rad( 0 );

	hgutil::B2R( plc0.B, cvmath::deg2rad( conf.pallet.direction_placedR_deg ), plc0.R );

	// Xbggpς
	//  findEmptyPlacingSlot()Ɉړ
	//conf.pallet.placing_slotB( model_id, slot_idx, 3 ) = 1;

	// ̊px̕␳
	plc0.torso_angle_rad = hgutil::optimizeTorsoAngleRad( plc0.R, plc0.side );

#	if 0
	cout << ">> check" << endl;
	cout << "\t[poseR]:"; hgutil::printPose3D( plc0.R );
	cout << "\t[poseB]:"; hgutil::printPose3D( plc0.B );
	cout << "\tL: " << (hg->isIKSolutionFoundL( plc0.R, plc0.torso_angle_rad ) ? "200" : "404") << endl;
	cout << "\tR: " << (hg->isIKSolutionFoundR( plc0.R, plc0.torso_angle_rad ) ? "200" : "404") << endl;
	cout << "<< check" << endl;
	getchar();
#	endif

	putch( '.' ), fflush( stdout );

	/* 1.̂] */
	{
		torso_angles_rad[0][0] = plc0.torso_angle_rad;
		ope_nodes.push_back(
			new JointOperation( hg,
				torso_angles_rad, conf.t_s(), true ) );
	}

	putch( '.' ), fflush( stdout );

	/* 2.uꏊ̏ */
	{
		R_tmp = plc0.R;
		R_tmp.position.z += 0.1;	 // ̐ݒuʒȕ

		if( !HandOperation::isIKSolutionFound( hg,
			plc0.side, R_tmp, plc0.torso_angle_rad ) )
			return sys::returnByError( -2, "[IK failed. (2)]" );

		ope_nodes.push_back( 
			new HandOperation( hg, 
				plc0.side, R_tmp, 1.5*conf.speed_base, true ) );
	}

	putch( '.' ), fflush( stdout );

	/* 3. */
	{
		if( !HandOperation::isIKSolutionFound( hg,
			plc0.side, plc0.R, plc0.torso_angle_rad ) )
			return sys::returnByError( -3, "[IK failed. (3)]" );

		ope_nodes.push_back( 
			new HandOperation( hg,
				plc0.side, plc0.R, 1.5*conf.speed_base, true ) );
	}

	putch( '.' ), fflush( stdout );

	/* 4.҂ */
	{
		ope_nodes.push_back( new SleepOperation( 0.2 ) );
	}

	putch( '.' ), fflush( stdout );

	/* 5.wJ */
	{
		ope_nodes.push_back( 
			new FingerDistanceOperation( hg, 
				plc0.side, 1000*( obj_size_m.w + conf.release_distance_m ),
				0.1, true ) );
	}

	putch( '.' ), fflush( stdout );

	/* 6.҂ */
	{
		ope_nodes.push_back( new SleepOperation( 0.2 ) );
	}

	putch( '.' ), fflush( stdout );

	/* 7. */
	{
		R_tmp = plc0.R;
		R_tmp.position.z += 0.1;	 // ̐ݒuʒȕ

		if( !HandOperation::isIKSolutionFound( hg,
			plc0.side, R_tmp, plc0.torso_angle_rad ) )
			return sys::returnByError( -7, "[IK failed. (7)]" );

		ope_nodes.push_back(
			new HandOperation( hg, 
				plc0.side, R_tmp, 1.5*conf.speed_base, true ) );
	}

	/* 8.p */
	{
		ope_nodes.push_back(
			new JointOperation( hg, 
				pnp::operation_t::pose_all_initial, conf.t_s(), true ) );
	}

	printf( "[OK]\n" );

	return 0;
}


#if 0
/**********************************************************
	̂̈ʒup𐢊En烍{bgnɕϊ
	 ObjectPoseEstimationGateRTC
**********************************************************/
void HiroPickerAndPlacer::W2R(
		const RTC::Pose3D &pose_objW,
		cvmath::Vector< CV_64F > &p_objR, cvmath::Matrix< CV_64F > &R_objR )
{
	cvmath::Vector< CV_64F > p_objW(3);
	
	// ̂̈ʒu(W: En, R: {bgn)
	p_objW(0) = pose_objW.position.x;
	p_objW(1) = pose_objW.position.y;
	p_objW(2) = pose_objW.position.z;
	
	p_objR.MxV( operation_t::R_WtoR, p_objW );
		// ]

	p_objR(0) += conf.worldR_m(0);
	p_objR(1) += conf.worldR_m(1);
	p_objR(2) += conf.worldR_m(2);
		// sړ

	// ̂̎p(W: En, R: {bgn)
	cvmath::Matrix< CV_64F > R_objW(3,3);
	cvmath::Quaternion< CV_64F > q[3], qtmp, qxyz;
	cvmath::Vector< CV_64F > axis(3);
	axis.setZero();	axis(0) = pose_objW.orientation.r;	q[0].fromAxis( axis );
	axis.setZero();	axis(1) = pose_objW.orientation.p;	q[1].fromAxis( axis );
	axis.setZero();	axis(2) = pose_objW.orientation.y;	q[2].fromAxis( axis );
	qtmp.QxQ( q[0], q[1] );
	qxyz.QxQ( qtmp, q[2] );
	qxyz.toR( R_objW );
		// EWn̉]s
	
	R_objR.MxM( operation_t::R_WtoR, R_objW );
		// Enł̉]s{bgnɕϊ
}
#endif