// -*- C++ -*-
/*!
 * @file  ObjectPoseEstimationGate.cpp
 * @brief ${rtcParam.description}
 * @date $Date$
 *
 * $Id$
 */

#include "ObjectPoseEstimationGate.h"


using std::cout;
using std::cerr;
using std::endl;


// Module specification
// <rtc-template block="module_spec">
static const char* objectposeestimationgate_spec[] =
  {
    "implementation_id", "ObjectPoseEstimationGate",
    "type_name",         "ObjectPoseEstimationGate",
    "description",       "${rtcParam.description}",
    "version",           "1.0.0",
    "vendor",            "AISL.TUT",
    "category",          "StateMachineGateway",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "0",
    "language",          "C++",
    "lang_type",         "compile",
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
ObjectPoseEstimationGate::ObjectPoseEstimationGate(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_inputIn("input", m_input),
    m_rec_resIn("RecognitionResult", m_rec_res),
    m_outputOut("output", m_output),
    m_RecognitionServicePort("RecognitionService"),
    m_MDServicePort("MDService"),
    m_ObjectPoseEstimationGateServicePort("ObjectPoseEstimationGateService")

    // </rtc-template>
		,
		opeg_svc_if( &md_svc )
{
}

/*!
 * @brief destructor
 */
ObjectPoseEstimationGate::~ObjectPoseEstimationGate()
{
}



RTC::ReturnCode_t ObjectPoseEstimationGate::onInitialize()
{
	cerr << ">> OPEG::onInitialize()" << endl;

  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  addInPort("input", m_inputIn);
  addInPort("RecognitionResult", m_rec_resIn);
  
  // Set OutPort buffer
  addOutPort("output", m_outputOut);
  
  // Set service provider to Ports
  m_ObjectPoseEstimationGateServicePort.registerProvider("ObjectPoseEstimationGateSvcIF", "ObjectPoseEstimationGateRTC::ObjectPoseEstimationGateService", opeg_svc_if);
  
  // Set service consumers to Ports
  m_RecognitionServicePort.registerConsumer("RecognitionService", "RecognitionService", rec_svc);
  m_MDServicePort.registerConsumer("MDService", "ModelDefinitionRTC::idl::MDService", md_svc);
  
  // Set CORBA Service Ports
  addPort(m_RecognitionServicePort);
  addPort(m_MDServicePort);
  addPort(m_ObjectPoseEstimationGateServicePort);
  
  // </rtc-template>
	
	init();

	cerr << "<< OPEG::onInitialize()" << endl;

  return RTC::RTC_OK;
}


RTC::ReturnCode_t ObjectPoseEstimationGate::onFinalize()
{
	cerr << ">> OPEG::onFinalize()" << endl;
	cerr << "<< OPEG::onFinalize()" << endl;
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t ObjectPoseEstimationGate::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ObjectPoseEstimationGate::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t ObjectPoseEstimationGate::onActivated(RTC::UniqueId ec_id)
{
	cerr << ">> OPEG::onActivated()" << endl;
	activate();
	cerr << "<< OPEG::onActivated()" << endl;
  return RTC::RTC_OK;
}


RTC::ReturnCode_t ObjectPoseEstimationGate::onDeactivated(RTC::UniqueId ec_id)
{
	cerr << ">> OPEG::onDeactivated()" << endl;
	deactivate();
	cerr << "<< OPEG::onDeactivated()" << endl;
  return RTC::RTC_OK;
}


RTC::ReturnCode_t ObjectPoseEstimationGate::onExecute(RTC::UniqueId ec_id)
{
	process();
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t ObjectPoseEstimationGate::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ObjectPoseEstimationGate::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ObjectPoseEstimationGate::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ObjectPoseEstimationGate::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t ObjectPoseEstimationGate::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void ObjectPoseEstimationGateInit(RTC::Manager* manager)
  {
    coil::Properties profile(objectposeestimationgate_spec);
    manager->registerFactory(profile,
                             RTC::Create<ObjectPoseEstimationGate>,
                             RTC::Delete<ObjectPoseEstimationGate>);
  }
  
};


