// -*- C++ -*-
/*!
 * @file  ObjectPoseEstimationGate.h
 * @brief ${rtcParam.description}
 * @date  $Date$
 *
 * $Id$
 */

#ifndef OBJECTPOSEESTIMATIONGATE_H
#define OBJECTPOSEESTIMATIONGATE_H

#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>
#include <rtm/idl/BasicDataTypeSkel.h>
#include <rtm/idl/ExtendedDataTypesSkel.h>
#include <rtm/idl/InterfaceDataTypesSkel.h>

// Service implementation headers
// <rtc-template block="service_impl_h">
#include "ObjectPoseEstimationGateRTCSVC_impl.h"

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="consumer_stub_h">
#include "VisionStub.h"
#include "ModelDefinitionCommonServiceStub.h"

// </rtc-template>

using namespace RTC;

/*!
 * @class ObjectPoseEstimationGate
 * @brief ${rtcParam.description}
 *
 */
class ObjectPoseEstimationGate
  : public RTC::DataFlowComponentBase
{
 public:
  /*!
   * @brief constructor
   * @param manager Maneger Object
   */
  ObjectPoseEstimationGate(RTC::Manager* manager);

  /*!
   * @brief destructor
   */
  ~ObjectPoseEstimationGate();

  // <rtc-template block="public_attribute">
  
  // </rtc-template>

  // <rtc-template block="public_operation">
  
  // </rtc-template>

  /***
   *
   * The initialize action (on CREATED->ALIVE transition)
   * formaer rtc_init_entry() 
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onInitialize();

  /***
   *
   * The finalize action (on ALIVE->END transition)
   * formaer rtc_exiting_entry()
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onFinalize();

  /***
   *
   * The startup action when ExecutionContext startup
   * former rtc_starting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onStartup(RTC::UniqueId ec_id);

  /***
   *
   * The shutdown action when ExecutionContext stop
   * former rtc_stopping_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onShutdown(RTC::UniqueId ec_id);

  /***
   *
   * The activated action (Active state entry action)
   * former rtc_active_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

  /***
   *
   * The deactivated action (Active state exit action)
   * former rtc_active_exit()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

  /***
   *
   * The execution action that is invoked periodically
   * former rtc_active_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

  /***
   *
   * The aborting action when main logic error occurred.
   * former rtc_aborting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onAborting(RTC::UniqueId ec_id);

  /***
   *
   * The error action in ERROR state
   * former rtc_error_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onError(RTC::UniqueId ec_id);

  /***
   *
   * The reset action that is invoked resetting
   * This is same but different the former rtc_init_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onReset(RTC::UniqueId ec_id);
  
  /***
   *
   * The state update action that is invoked after onExecute() action
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onStateUpdate(RTC::UniqueId ec_id);

  /***
   *
   * The action that is invoked when execution context's rate is changed
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onRateChanged(RTC::UniqueId ec_id);


 protected:
  // <rtc-template block="protected_attribute">
  
  // </rtc-template>

  // <rtc-template block="protected_operation">
  
  // </rtc-template>

  // DataInPort declaration
  // <rtc-template block="inport_declare">
  TimedStringSeq m_input;
  /*!
   */
  InPort<TimedStringSeq> m_inputIn;
  TimedDoubleSeq m_rec_res;
  /*!
   */
  InPort<TimedDoubleSeq> m_rec_resIn;
  
  // </rtc-template>


  // DataOutPort declaration
  // <rtc-template block="outport_declare">
  TimedStringSeq m_output;
  /*!
   */
  OutPort<TimedStringSeq> m_outputOut;
  
  // </rtc-template>

  // CORBA Port declaration
  // <rtc-template block="corbaport_declare">
  /*!
   */
  RTC::CorbaPort m_RecognitionServicePort;
  /*!
   */
  RTC::CorbaPort m_MDServicePort;
  /*!
   */
  RTC::CorbaPort m_ObjectPoseEstimationGateServicePort;
  
  // </rtc-template>

  // Service declaration
  // <rtc-template block="service_declare">
  /*!
   */
  ObjectPoseEstimationGateServiceSVC_impl opeg_svc_if;
  
  // </rtc-template>

  // Consumer declaration
  // <rtc-template block="consumer_declare">
  /*!
   */
  RTC::CorbaConsumer<RecognitionService> rec_svc;
  /*!
   */
  RTC::CorbaConsumer<ModelDefinitionRTC::idl::MDService> md_svc;
  
  // </rtc-template>

 private:
  // <rtc-template block="private_attribute">
	// ܂łɌ̑
	int found_obj_count;
	// Ώۂ̕ID
  int model_id_to_be_tried;
	// 蒆CvtO
	bool in_estimation, request_to_estimate;

#	ifdef OPEG_FIND_ONE_OBJECT
#	else

	// 茋ʃV[PX1̗̂vf
	static const int REC_RES_N_ELEM = 20;

	// 茋(m_rec_res)ێpV[PX
	TimedDoubleSeq result_q;

	// result_qpf[^
	// ̐Ō̐ƑΏە̃CfbNX
	int n_result, result_idx;

#	endif

  // </rtc-template>

  // <rtc-template block="private_operation">
	int process();
	int init();
	int activate();
	int deactivate();

	int processRequestToEstimate( std::string &output );
	int processResult( std::string &output );

	int queueResult();
	int dequeueResult();
	int deleteResult();
  // </rtc-template>

};


extern "C"
{
  DLL_EXPORT void ObjectPoseEstimationGateInit(RTC::Manager* manager);
};

#endif // OBJECTPOSEESTIMATIONGATE_H
