// -*- C++ -*-
/*!
 * @file  PathPlannerV2.h
 * @brief ModuleDescription
 * @date  $Date$
 *
 * $Id$
 */

#ifndef PATHPLANNERV2_H
#define PATHPLANNERV2_H

#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>
#include <rtm/idl/BasicDataTypeSkel.h>
#include <rtm/idl/ExtendedDataTypesSkel.h>
#include <rtm/idl/InterfaceDataTypesSkel.h>
//#include <rtm/idl/IIS.h>
//#include <rtm/idl/MRFC.h>
//#include <rtm/idl/TUT.h>

// Service implementation headers
// <rtc-template block="service_impl_h">

// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="consumer_stub_h">
#include "MRFCStub.h"
#include "IIS2Stub.h"
#include "ImageDataStub.h"
#include "UtilTimedImageData.h"

// </rtc-template>
#ifndef PI
#define PI (3.1415926)
#endif
#define ABS(x) ((x) > 0 ? (x):-(x))
#define RAD_to_DEG (180.0/PI)
#define DEG_to_RAD (PI/180.0)


#include "base_data.h"
#include "tree_new.h"
#include "obst_info.h"
#include "draw.h"
#include "stack.h"
#include "func.h"
#include "robot_sound.h"
#include <vector>
#include "level_set_method.h"
#if FP_TEST
#include "save.h"
#endif
#include <mmsystem.h>
#include <windows.h>

using namespace RTC;
using namespace IIS;
//using namespace MRFC;
using namespace TUT;

/*!
 * @class PathPlannerV2
 * @brief ModuleDescription
 *
 */
class PathPlannerV2
  : public RTC::DataFlowComponentBase
{
 public:
  /*!
   * @brief constructor
   * @param manager Maneger Object
   */
  PathPlannerV2(RTC::Manager* manager);

  /*!
   * @brief destructor
   */
  ~PathPlannerV2();

  // <rtc-template block="public_attribute">
  
  // </rtc-template>

  // <rtc-template block="public_operation">
  
  // </rtc-template>

  /***
   *
   * The initialize action (on CREATED->ALIVE transition)
   * formaer rtc_init_entry() 
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onInitialize();

  /***
   *
   * The finalize action (on ALIVE->END transition)
   * formaer rtc_exiting_entry()
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onFinalize();

  /***
   *
   * The startup action when ExecutionContext startup
   * former rtc_starting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onStartup(RTC::UniqueId ec_id);

  /***
   *
   * The shutdown action when ExecutionContext stop
   * former rtc_stopping_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onShutdown(RTC::UniqueId ec_id);

  /***
   *
   * The activated action (Active state entry action)
   * former rtc_active_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

  /***
   *
   * The deactivated action (Active state exit action)
   * former rtc_active_exit()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

  /***
   *
   * The execution action that is invoked periodically
   * former rtc_active_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
   virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

  /***
   *
   * The aborting action when main logic error occurred.
   * former rtc_aborting_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onAborting(RTC::UniqueId ec_id);

  /***
   *
   * The error action in ERROR state
   * former rtc_error_do()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onError(RTC::UniqueId ec_id);

  /***
   *
   * The reset action that is invoked resetting
   * This is same but different the former rtc_init_entry()
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onReset(RTC::UniqueId ec_id);
  
  /***
   *
   * The state update action that is invoked after onExecute() action
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onStateUpdate(RTC::UniqueId ec_id);

  /***
   *
   * The action that is invoked when execution context's rate is changed
   * no corresponding operation exists in OpenRTm-aist-0.2.0
   *
   * @param ec_id target ExecutionContext Id
   *
   * @return RTC::ReturnCode_t
   * 
   * 
   */
  // virtual RTC::ReturnCode_t onRateChanged(RTC::UniqueId ec_id);


 protected:
  // <rtc-template block="protected_attribute">
  
  // </rtc-template>

  // <rtc-template block="protected_operation">
  
  // </rtc-template>

  // Configuration variable declaration
  // <rtc-template block="config_declare">
  /*!
   * 
   * - Name:  GOAL_AREA
   * - DefaultValue: 0.300
   */
  double m_GOAL_AREA;
  /*!
   * 
   * - Name:  LOOP_TIME
   * - DefaultValue: 0.500
   */
  double m_LOOP_TIME;
  /*!
   * 
   * - Name:  ROBOT_RADIUS
   * - DefaultValue: 0.310
   */
  double m_ROBOT_RADIUS;
  /*!
   * 
   * - Name:  MOVING_OBSTACLE_RADIUS
   * - DefaultValue: 0.200
   */
  double m_MOVING_OBSTACLE_RADIUS;
  /*!
   * 
   * - Name:  MOTION_SET_FILE
   * - DefaultValue: MotionSet.mtn
   */
  std::string m_MOTION_SET_FILE;
  /*!
   * 
   * - Name:  TREAD
   * - DefaultValue: 0.3325
   */
  double m_TREAD;
  /*!
   * 
   * - Name:  ROBOT_ACCELERATION
   * - DefaultValue: 0.400
   */
  double m_ROBOT_ACCELERATION;
  /*!
   * 
   * - Name:  ROBOT_MAX_SPEED
   * - DefaultValue: 0.500
   */
  double m_ROBOT_MAX_SPEED;
  /*!
   * 
   * - Name:  SOUND_SET
   * - DefaultValue: 2
   */
  int m_SOUND_SET;
  /*!
   * 
   * - Name:  USE_WAYPOINT_PORT
   * - DefaultValue: 0
   */
  int m_USE_WAYPOINT_PORT;
  /*!
   * 
   * - Name:  USE_LOCALIZATION_PORT
   * - DefaultValue: 1
   */
  int m_USE_LOCALIZATION_PORT;
  /*!
   * 
   * - Name:  USE_PEOPLE_TRACKING
   * - DefaultValue: 0
   */
  int m_USE_PEOPLE_TRACKING;
  /*!
   * 
   * - Name:  USE_MAP_SERVICE
   * - DefaultValue: 1
   */
  int m_USE_MAP_SERVICE;
  /*!
   * 
   * - Name:  USE_PEOPLE_SERVICE
   * - DefaultValue: 1
   */
  int m_USE_PEOPLE_SERVICE;
  /*!
   * 
   * - Name:  USE_WAYPOINT_HEADING
   * - DefaultValue: 0
   */
  int m_USE_WAYPOINT_HEADING;
  /*!
   * 
   * - Name:  USE_VIDEO
   * - DefaultValue: 0
   */
  int m_USE_VIDEO;
  // </rtc-template>
  int m_BACK_STEP;

  float m_setpoint1;
  float m_setpoint2;

  float m_check1;
  float m_check2;
  float m_check3;
  float m_check4;

  // DataInPort declaration
  // <rtc-template block="inport_declare">
  IIS::TimedPose2D m_robot_pose;
  /*!
   */
  InPort<IIS::TimedPose2D> m_robot_poseIn;
  IIS::TimedVelocity2D m_robot_velocity;
  /*!
   */
  InPort<IIS::TimedVelocity2D> m_robot_velocityIn;
  IIS::TimedPoseVel2DSeq m_subgoal_waypoint;
  /*!
   */
  InPort<IIS::TimedPoseVel2DSeq> m_subgoal_waypointIn;
  RTC::TimedString m_in_message;
  InPort<RTC::TimedString> m_in_messageIn;
  
  // </rtc-template>


  // DataOutPort declaration
  // <rtc-template block="outport_declare">
  IIS::TimedVelocity2D m_robot_control;
  /*!
   */
  OutPort<IIS::TimedVelocity2D> m_robot_controlOut;
  TUT::TimedImageData m_planning_img;
  /*!
   */
  OutPort<TUT::TimedImageData> m_planning_imgOut;

  // message output
  RTC::TimedString m_message;
  OutPort<RTC::TimedString> m_messageOut;
  
  // </rtc-template>

  // CORBA Port declaration
  // <rtc-template block="corbaport_declare">
  /*!
   */
  RTC::CorbaPort m_RelativeMapServicePort;
  /*!
   */
    
  // </rtc-template>

  // Service declaration
  // <rtc-template block="service_declare">
  
  // </rtc-template>

  // Consumer declaration
  // <rtc-template block="consumer_declare">
  /*!
   */
  RTC::CorbaConsumer<MRFC::RelativeMapService> m_RelativeMapService;
  /*!
   */
  
  
  // </rtc-template>

 private:
	 //Original
	 void exit_robot();//Robot Stop
	 float _robotToGoalDist(void);
	 void drawing_when_error();
	 void drawing_whole();
	 int dummy;
	 void make_motion(MOTION_CONTROL *output,MOTION_CONTROL *input1);
	 void make_motion2(MOTION_CONTROL *output);
	 void error_action(float velocity, float radius);
	 bool check_LM_connection();

	 // function for execute
	 void pathArrayControl(void);
	 void send_next_control();
	 void memory_consumption();
	 void get_robot_state();
	 void get_robot_velocity();
	 void predict_and_store_robot_pose();
	 void get_data_waypoint();
	 void get_data_waypoint2();
	 void escape_from_collision();
	 void get_data_waypoint_global();
	 float read_distance();
	 float read_distance_wall_right();
	 float read_distance_wall_left();
	 void rotation(float sudut,int status);
	 

	 //For image storage
	IplImage *conter;
	IplImage *simulator;
	IplImage *LocalImg;
	IplImage *Gauss_f32 , *Gauss_u8;
	CvVideoWriter *writer;
	IplImage *dt_32;

	IplImage *window_output;

	char window_title_simu[BUF];
	char window_title_Local[BUF];
	
	MOTION_CONTROL motion,motion1;//Import Motion
	
	vector<TREE> leaf;
	TREE path[MAXNODE + MAXSTEP], tree[MAXNODE];
	Position goal;

	/***** Datalog relations *****/
#if FP_TEST//Data storage
	Save_struct saveFile;
#endif

	unsigned char **LocalMap;
	/*** Time calculation ***/
	DWORD end3, total3, max3, draw_time;
	DWORD time_tmp;
	int step;
	int n;
	bool flag;
	float xs[4];//0:nan 1:x-coordinate 2:y-coordinate 3:Direction
	float xg[4];//0:nan 1:x-coordinate 2:y-coordinate 3:Direction
	float xg_global_sub[4] , xg_local_sub[4];
	int path_depth;
	int closest_node;
	int keep_velocity;

	R_OBJE robot;
	ROBOT_STATE robot_state_tmp;
	DATA_LOGGER robot_data;
	M_OBJE *move_obje;
	M_OBJE *array_Mobje[OBJECT_MAX_NUM];
	short NumOfMobje;
	DWORD simu_start, simu_end, calc_Draw_ave, calc_Draw_max;
	double _cos_rad, _sin_rad;
	unsigned short warning_flag;//Collision detection, time-over detection flag
	int trace_id;
	unsigned char orientation;
	unsigned char collision_state;
	float collision_direction;
	Position collision_director;

	int waypoint_trace_complete_num;//waypoint available when using.
	int waypoint_data_length;//
	int waypoint_pointer;
	int waypoint_state_change;
	float temp_way_x,temp_way_y;
	float waypoint_direction_temp;
	int waypoint_end_flag;

	int previous_trace_flag;


	/*People tracking*/
	RECEIVE_OBST_DATA *data_people;
	short m_counter;

	MRFC::TimedPeopleTrackingData * people_tracking_data;

	int activate_count;
	bool pre_arrive_flag;
	bool pre_waypoint_end_flag;
	bool initial_waypoint;
	float temp_rad,temp_vel;
	int counter_stop_waypoint;
	float robotToGoalDistance;
	bool after_collision;
	bool first_start;

	int states,count_geser;
	float jarak,jarak_dinding;
	std::string kata;



	/* Debugging output */
#ifdef ___DEBUG_FILE___
	FILE *debug_file;
	char debug_file_name[BUF];
#endif
  // <rtc-template block="private_attribute">
  
  // </rtc-template>

  // <rtc-template block="private_operation">
  
  // </rtc-template>

};


extern "C"
{
  DLL_EXPORT void PathPlannerV2Init(RTC::Manager* manager);
};

#endif // PATHPLANNERV2_H
