/*********************************************************/
/* httpc.cpp                                             */
/* @ Author: J.Sugiyama@AISL.TUT                         */
/* @ Date  : Mar.11.2012                                 */
/* @ Release note:                                       */
/*   Corrected a bug (POST timing confliction)           */ 
/*                                           Mar.11.2012 */
/*********************************************************/
#include "httpc.h"
#include "protocol.h"
#include <sstream>


using namespace std;


HTTPC::HTTPC()
	: is_initialized( false ), templ_html_len( 0 ), templ_html( NULL )
{
	//cout << ">> HTTPC::HTTPC()" << endl;
	//cout << "<< HTTPC::HTTPC()" << endl;
}


HTTPC::~HTTPC()
{
	//cout << ">> HTTPC::~HTTPC()" << endl;
	if( is_initialized ) finalize();
	//cout << "<< HTTPC::~HTTPC()" << endl;
}


int HTTPC::initialize( unsigned short tcp_port_no, const char *template_html_path )
{
	if( is_initialized ) return -1;


	cout << "Initializing data...................";
	kill_switch = false;
	data_posted.initialize( true, false );
	data_updated.initialize( true, false );
	cs.initialize();

	pushing = false;

	{
		int ch;
		string templ_html_tmp;

		FILE *fp = fopen( template_html_path, "rb" );
		if( !fp )
			return sys::returnByError( -2,
				"HTTPC::initialize(): cannot open template HTML.", true );
		
		while( (ch = fgetc( fp )) != EOF )
			templ_html_tmp.push_back( ch );
		fclose( fp );

		templ_html_len = int( templ_html_tmp.length() );
		templ_html = new char [templ_html_len+1];
		strcpy( templ_html, templ_html_tmp.c_str() );
	}
  cout << "[OK]" << endl;

	//cout << templ_html << endl;

	cout << "Opening server socket...............";
  if( server.open() )
    return sys::returnByError( -3,
			"HTTPC::initialize(): cannot open socket.", true );
  cout << "[OK]" << endl;
	
	cout << "Setting acceptable client address...";
	if( server.setAddressINet( (unsigned long) INADDR_ANY, tcp_port_no ) )
    return sys::returnByError( -4,
			"HTTPC::initialize(): cannot set client address.", true );
  cout << "[OK]" << endl;

	cout << "Binding server socket...............";
  if( server.bind() )
    return sys::returnByError( -5,
			"HTTPC::initialize(): cannot connect to server.", true );
  cout << "[OK]" << endl;

	cout << "Lisning with server socket..........";
  if( server.listen( 0 ) )
    return sys::returnByError( -6,
			"HTTPC::initialize(): cannot listen.", true );
  cout << "[OK]" << endl;

	cout << "Starting server thread..............";
	thread.start( process, this );
  cout << "[OK]" << endl;

	is_initialized = true;

	cout << "Server started successfully" << endl;

	return 0;
}


int HTTPC::finalize()
{
	if( !is_initialized ) return -1;

	kill_switch = true;

	server.close();

	//data_posted.reset();
	//data_updated.post();

	thread.stop();

	data_posted.finalize();
	data_updated.finalize();
	cs.finalize();

	templ_html_len = 0;
	if( templ_html )
		delete [] templ_html;
	templ_html = NULL;

	pushing = false;

	is_initialized = false;
	
	return 0;
}

/*
void HTTPC::setHTML(
	const string &state, const string &next_status, int n_trans )
{
	this->state = state;
	this->next_status = next_status + ", \"Abort\"";

	ostringstream oss;

	int btn_i;
	for( btn_i = 0; btn_i < n_trans; ++ btn_i )
		oss << "<input type=\"submit\" name=\"state\" id=\"func" << btn_i << "\" onclick=\"disableButton(this); return false;\" /><br>";

	// Aboart {^
	oss << "<br><input type=\"submit\" name=\"state\" id=\"func" << btn_i << "\" onclick=\"disableButton(this); return false;\"/>";

	input_tags = oss.str();
}
*/


void HTTPC::postDataUpdated()
{
	data_updated.post();
}


bool HTTPC::waitForPOST()
{
	if( WAIT_OBJECT_0 != data_posted.wait( 0 ) )
		return false;

	data_posted.reset();
	return true;
}


void HTTPC::getHTMLParameter( string &state, string &next_status )
{
	cs.enter();
	state = this->state;
	next_status = this->next_status;
	cs.exit();
}


/**
	HTMLɖߍރp[^̐ݒ
	@	state:           Ԗ
	@	next_status:     Ԗ
	@	disabled_status: Jڃ{^̗L/
*/
void HTTPC::setHTMLParameter(
	const string &state,
	const vector< string > &next_status,
	const vector< bool > &disabled_status )
{
	ostringstream oss;
	int nx_i, nx_n = (int) next_status.size();

	cs.enter();

	// 
	this->state = '\"' + state + '\"';

	// {^p^O
	for( nx_i = 0; nx_i < nx_n; ++ nx_i )
		oss
			<< "<input "
			<< ( disabled_status.at(nx_i) ? "disabled " : "" )
			<< "type=\"submit\" name=\"" << state << "\" value=\"" << next_status.at(nx_i) <<  "\" onsubmit=\"return processOnce(this);\"/><br>";
	oss << "<br><input type=\"submit\" name=\"" << state << "\" value=\"Abort\" onsubmit=\"return processOnce(this);\"/>";
		// Aboart {^
	input_tags = oss.str();

	cs.exit();
}


void HTTPC::getPostedData( string &posted )
{
	cs.enter();	posted = posted_data; cs.exit();
}


unsigned int __stdcall HTTPC::process( void *httpc_vp )
{
	HTTPC &httpc = *((HTTPC *) httpc_vp);
	bool &kill_switch = httpc.kill_switch;
	const int &templ_html_len = httpc.templ_html_len;
	const char *templ_html = httpc.templ_html;
	sys::TCPSocket
		&server = httpc.server,
		&to_client = httpc.to_client;

	vector< char > &buf = httpc.buf;
	string
		&request = httpc.request,
		&response = httpc.response;

	sys::ConsoleController cc;

	HTTP http;

	const size_t LSQ = strlen("/state-query+");

	//cc.clear( cc.BLACK, 0 );

	while( !kill_switch ) {
		while( !server.accept( to_client ) ) {
			//cc.setColor();
			//cc.setPosition();
			//cout << "HTTPC::process(): accept; "; to_client.printINetAddr();

			/* NGXg̎M */
			if( httpc.receiveRequest() )
				break;

			/* NGXg̕ */
			if( http.parseRequest( request ) )
				break;

			//cerr << "[Request: " << http.getReqMethod() << http.getReqPath() << "]" << endl;


			/* X|X̐ */
			buf.clear();
			buf.resize( templ_html_len + 1024 );

			switch( http.getReqMethodId() ){
			case HTTP::MET_ID_GET:
				// GET\bh
				if( http.getReqPath() == "/favicon.ico" )
					http.set404();
				else if( http.getReqPath().substr(0, LSQ) == "/state-query+" ) {
					
					/*
					cout
						<< "GET:  [" << http.getReqPath() << "] "
						<< "(" << http.getReqPath().substr(LSQ) << ")"
						<< "(" << httpc.state.substr( 1, httpc.state.length()-2 ) << ")"
						<< endl;
					*/

					if( httpc.pushing
						|| http.getReqPath().substr(LSQ) != httpc.state.substr( 1, httpc.state.length()-2 ) ) {
						// push vꍇC܂͌ԖɑႪꍇHTMLXV
						// httpc.pushing ̃`FbN͕sv (Mar.11.2012)
						httpc.createHTML();
						http.set200();
						httpc.pushing = false;
					}
					else {
						http.set404();
					}
				}
				else {
					httpc.createHTML();
					http.set200( &buf[0] );
				}
				break;

			case HTTP::MET_ID_POST:
				// POST\bh
				//cout << "POST: [" << http.getReqMsgBody() << "]" << endl;

				httpc.cs.enter();
				httpc.posted_data = http.getReqMsgBody();
				httpc.cs.exit();

				httpc.data_posted.post();

				httpc.data_updated.wait();
				httpc.data_updated.reset();

				httpc.createHTML();
				http.set200( &buf[0] );

				break;
			default:
				http.set404();
				break;
			}

			// ̐
			http.createResponse( response );

			//cc.setPosition( 0, 2 );
			//cerr << "[Responce: " << http.getResStatMsg() << "]" << endl;

			/* X|X̑M */
			if( httpc.sendResponse() )
				break;
			
			//cc.setPosition( 0, 3 );
			//cout << "HTTPC::process(): connection close; "; to_client.printINetAddr();
			to_client.close();
		}
		cc.setColor( cc.YELLOW );
		//cc.setPosition( 0, 3 );
		//cout << "HTTPC::process(): connection close; "; to_client.printINetAddr();
		cc.setColor();
		to_client.close();
	}
	server.close();

	return 0;
}


int HTTPC::receiveRequest()
{
	int length;

	/* NGXg̎擾 */
	buf.resize( 1024 );
			
	// ...\r\n\r\n...ǂݍ
	size_t rnrn_idx;
	request = "";
	do {
		length = to_client.recv( &buf[0], buf.size()-1 );
		//printf( "%d,%d\n", length, strlen( &buf[0] ) );
		if( length == 0 ) break;
		else if( length < 0 )
			return sys::returnByError( -1,
				"HTTPC::receiveRequest(): failed." );
		buf[length] = '\0';
		request += &buf[0];

		rnrn_idx = request.find( "\r\n\r\n" );
	} while( rnrn_idx == string::npos );
	
	/*
	printf( "%02x%02x%02x%02x\n",
		rbuf_str.at(rnrn_idx), rbuf_str.at(rnrn_idx+1),
		rbuf_str.at(rnrn_idx+2), rbuf_str.at(rnrn_idx+3) );
	*/

	// NGXg{fBɓǂݐi߂(܂c肪ꍇ)
	// \r\n\r\n ܂ł̗vfContent-Lengthǂݍ
	size_t content_len_idx = request.find( "Content-Length: " );
	if( content_len_idx != string::npos ) {
		int content_len;
		string content_len_line = request.substr(
			content_len_idx, request.find( "\r\n", content_len_idx ) );
		//cout << content_len_line << endl;
		sscanf( content_len_line.c_str(), "Content-Length: %d", &content_len );
		//cout << "Content-Length: " << content_len << endl;
		
		// ׂẴf[^ǂ݂ĂȂꍇ
		while( (rnrn_idx + 4 + content_len) != request.length() ) {
			length = to_client.recv( &buf[0], buf.size()-1 );
			buf[length] = '\0';

			if( length <= 0 )
				return sys::returnByError( -1,
					"HTTPC::receiveRequest(): failed." );
			else if( length > 0 )
				request += &buf[0];
		}
	}

	return 0;
}


int HTTPC::createHTML()
{
	// f[^擾藧ĂKv
	// ԑJڎɂ"̃R[h"setHTMLParameter()sƂ悢

	//httpc.setHTML( "\"I\"", "\"AGV outward\"", 1 );

	cs.enter();
	sprintf( &buf[0], templ_html,
		state.c_str(), input_tags.c_str() );
	cs.exit();

	return 0;
}


int HTTPC::sendResponse()
{
	int length = to_client.send( response );
	if( length != response.length() )
		return sys::returnByError( -1,
			"HTTPC::sendResponse(): failed." );

	return 0;
}