#include "mime.h"


using namespace std;


const MIME::entry_t MIME::ENTRY[MIME_N] = {
	{ "html",       "text/html" },	// HTML
	{ "htm",        "text/html" },	// HTML
	{ "xml",        "text/xml" },		// XML
	{ "txt",        "text/plain" },	// Test
	{ "csv",        "text/comma-separated-values" },	// CSV
	{ "png",        "image/png" },	// PNG image
	{ "jpg",        "image/jpeg" },	// JPEG image
	{ "jpeg",       "image/jpeg" },	// JPEG image]
	{ "urlencoded", "application/x-www-form-urlencoded" },
	{ "",           "application/octet-stream" }		// Unknown
};


/**
	Get mime type string from suffix
	<I>	path	:
	<O>	id		:
*/
const string &MIME::get( const string &path, int *id )
{
#if WXDEBUG == 1
	cerr << "MIME::get()" << endl;
#endif

	// Search the last '.' of path.
	size_t suffix_start = path.find_last_of( "." );

#if WXDEBUG == 1
	cerr << path << endl;
#endif

	if( suffix_start == std::string::npos ) {
#if WXDEBUG == 1
		cerr << "Missing '.'" << endl;
#endif
		if( id )	*id = MIM_ID_UNKNOWN;
		return MIME::ENTRY[MIM_ID_UNKNOWN].description;
	}

	// The suffix_start + 1 is index where the suffix really starts.
	string suffix = path.substr(suffix_start + 1);

#if WXDEBUG == 1
	cerr << suffix << endl;
#endif

	int i;
	for( i = 0; i < MIME_N - 1; i ++ ) {
		// Firstly, we attempt to compare the first character of suffix.
		if(ENTRY[i].suffix.at(0) != suffix.at(0))
			continue;

		// Ant then, we attempt to compare the whole of suffix.
		if(ENTRY[i].suffix == suffix)
			break;
	}

#if WXDEBUG == 1
	cerr << ENTRY[i].description << endl;
#endif

	if(id)	*id = i;
	return ENTRY[i].description;
}


const string &MIME::get( int id )
{
#if WXDEBUG == 1
	cerr << "MIME::get()" << endl;
#endif

	if(id < MIM_ID_HTML && MIM_ID_JPEG < id)
		return ENTRY[MIM_ID_UNKNOWN].description;
	return ENTRY[id].description;
}