#include "protocol.h"
#include <sstream>
#include <iostream>
#include <Windows.h>

using namespace std;


const HTTPMethod HTTP::METHOD[ METHOD_N ] = {
	HTTPMethod( MET_ID_GET,     "GET"  ),
	HTTPMethod( MET_ID_HEAD,    "HEAD" ),
	HTTPMethod( MET_ID_PUT,     "PUT"  ),
	HTTPMethod( MET_ID_POST,    "POST" ),
	HTTPMethod( MET_ID_UNKNOWN, ""     )
};

const HTTPVersion HTTP::VERSION[ VERSION_N ] = {
	HTTPVersion( VER_ID_HTTP_10, "HTTP/1.0" ),	
	HTTPVersion( VER_ID_HTTP_11, "HTTP/1.1" ),
	HTTPVersion( VER_ID_UNKNOWN, ""         )
};

const HTTPResponse HTTP::RESPONSE[ RESPONSE_N ] = {
	HTTPResponse( 200, "OK"                    ),
	HTTPResponse( 204, "No Content"            ),
	HTTPResponse( 400, "Bad Request"           ),
	HTTPResponse( 403, "Forbidden"             ),
	HTTPResponse( 404, "Not Found"             ),
	HTTPResponse( 405, "Method Not Allow"      ),
	HTTPResponse( 500, "Internal Server Error" ),
	HTTPResponse( 501, "Not Implemented"       ),
	HTTPResponse( 503, "Server Unavairable"    ),
	HTTPResponse( 0  , ""                      )
};

const HTTPHeader HTTP::HEADER[HTTP::HEADER_N] = 
{
	HTTPHeader( HDR_ID_ACCE,    "Accept"          ),
	HTTPHeader( HDR_ID_HOST,    "Host"            ),
	HTTPHeader( HDR_ID_CONN,    "Connection"      ),
	HTTPHeader( HDR_ID_CLEN,    "Content-Length"  ),
	HTTPHeader( HDR_ID_CTYP,    "Content-Type"    ),
	HTTPHeader( HDR_ID_DATE,    "Date"            ),
	HTTPHeader( HDR_ID_AENC,    "Accept-Encoding" ),
	HTTPHeader( HDR_ID_ALNG,    "Accept-Language" ),
	HTTPHeader( HDR_ID_ACHR,    "Accept-Charset"  ),
	HTTPHeader( HDR_ID_USRA,    "User-Agent"      ),
	HTTPHeader( HDR_ID_CCTR,    "Cache-Control"   ),
	HTTPHeader( HDR_ID_RFRR,    "Referer"         ),
	HTTPHeader( HDR_ID_ORIG,    "Origin"         ),
	HTTPHeader( HDR_ID_UNKNOWN, ""                )
};


const string HTTP::DELIMITER = "\r\n\r\n";
const string HTTP::ENDL = "\r\n";

/**
	Parse request
	@	<I>	request	:	the pointer to request string
	@	<O>	req_set	:	the reference of request msg_t
*/
int HTTP::parseRequest( const string &req )
{
#if WXDEBUG == 1
	cerr << "HTTP::parseRequest(0)" << endl;
#endif

	// request -> request line + header
	string req_line, req_header;
	if( separateReq( req, req_line, req_header ) < 0 )
		return -1;

	if( parseReqLine( req_line ) < 0 )
		return -2;

	if( parseReqHeader( req_header ) < 0 )
		return -3;

	return 0;
}


#if 0
/**
	Process the GET method
	@	<I>	dm
*/
int HTTP::processGet( const string &uri )
{
#if WXDEBUG == 1
	cerr << "HTTP::processGet()" << endl;
#endif

#if 0
	if( uri != "/index.htm" && uri != "/index.html" ) {
		// The "404 Not Found" has the "NOT FOUND" html.
		res_set.idx = RES_ID_NOT_FOUND;
		setHTMLTemplToResBody();
		return -1;
	}

	const size_t size = 0;

#if WXDEBUG == 1
	cerr << "\tsize = " << size << endl;
#endif

	if( size ) {
		res_set.idx = RES_ID_OK;
		setHTMLTemplToResBody();
	}
	else {
		// The "204 No Content" must NOT have the message body.
		res_set.idx = RES_ID_NO_CONTENT;
		setResHdr( NULL, NULL );
	}
#else
	// ̏ꍇ͕K templ.html ̓eԂ
	res_set.idx = RES_ID_OK;
	setHTMLTemplToResBody();
	
#endif

	return 0;
}


/**
	Process the Post method
	@	<I>	body
*/
int HTTP::processPost( const string &body )
{
#if WXDEBUG == 1
	cerr << "HTTP::processPut()" << endl;
#endif

	if( true ) {
		// The "404 Not Found" has the "NOT FOUND" html.
		res_set.idx = RES_ID_NOT_FOUND;
		setHTMLTemplToResBody();
		return -1;
	}

	/*
	if( true )	res_set.idx = RES_ID_OK;
	else								res_set.idx = RES_ID_NO_CONTENT;

	setResHdr(NULL, NULL);

#if WXDEBUG == 1
	for(size_t i = 0; i < body.size(); i ++)
		fprintf(stderr, "[body] %02x\n", body.at(i));
#endif
	
	dit->set(body);

#if WXDEBUG == 1
	cerr << "<< HTTP::processPut()" << endl;
#endif
	*/

	return 0;
}


/**
	Process unknown methods
*/
int HTTP::processUnknown()
{
#if WXDEBUG == 1
	cerr << "HTTP::processUnKnown()" << endl;
#endif

	res_set.idx = RES_ID_NOT_IMPL;
	setHTMLTemplToResBody();
	return 0;
}


/**
	Create response strings.
	@	return	:	with message body: 1/ without: 0
*/
bool HTTP::createResponse( string &buf )
{
#if WXDEBUG == 1
	cerr << "HTTP::createResponse()" << endl;
#endif

	ostringstream oss;

	oss
		<< "HTTP/1.1 " << RESPONSE[res_set.idx].getStatusCode() << " "
		<< RESPONSE[res_set.idx].getStatusMessage() << ENDL;

	for( int i = 0; i < HEADER_N - 1; i ++ ) {
		if( res_set.header[i].length() ){
			oss << HEADER[i].getStr() << ": " << res_set.header[i] << ENDL;
		}
	}

	oss << ENDL;

	buf = oss.str();

#if WXDEBUG == 1
	cerr
		<< "\t[Response]" << endl
		<< buf << endl;

	if( res_set.body.size() ){
		cerr
			<< "\t[Message body]" << endl;
		cerr << res.set.body << endl;
	}
#endif

	return res_set.body.size() ? true : false;
}
#endif


/**
	Create response strings.
	@	return	:	with message body: 1/ without: 0
*/
int HTTP::createResponse( string &buf )
{
	ostringstream oss;

	oss
		<< "HTTP/1.1 " << RESPONSE[res_set.idx].getStatusCode() << " "
		<< RESPONSE[res_set.idx].getStatusMessage() << ENDL;

	for( int i = 0; i < HEADER_N - 1; i ++ ) {
		if( res_set.header[i].length() ){
			oss << HEADER[i].getStr() << ": " << res_set.header[i] << ENDL;
		}
	}

	oss << ENDL;

	if( getResContLen() ) {
		oss << res_set.body;
	}

	buf = oss.str();

	return (int) buf.length();
}


int HTTP::setResStatCode( int code )
{
	for( int i = 0; i < RESPONSE_N; ++ i ) {
		if( RESPONSE[ i ].getStatusCode() == code ) {
			res_set.idx = i;
			return 0;
		}
	}
	return -1;
}


void HTTP::setResContLen( size_t size )
{
	ostringstream oss;
	oss << size;
	res_set.header[HDR_ID_CLEN] = oss.str();
}


/**
	Separate riquest strings into request line, header.
	(Message body is not included.)
	@	<I>	req_line	:	the reference of the request line
*/
int HTTP::separateReq( const string &req, string &req_line, string &req_header )
{
#if WXDEBUG == 1
	cerr << "HTTP::separateReq()" << endl;
#endif

	/* Separate the request string into the request line and header. */
	// Request line
	size_t rl_end_idx;
	if( (rl_end_idx = req.find(ENDL, 0)) == string::npos )	return -1;
	req_line = req.substr( 0, rl_end_idx );

#if WXDEBUG == 1
	cerr << "\t[Req line]   " << req_line << endl;
#endif

	// Request header
	size_t rh_top_idx = rl_end_idx + ENDL.length(), rh_end_idx;

	// If there is no header
	if(req.find(ENDL, rh_top_idx) == rh_top_idx) return 0;

	if((rh_end_idx = req.find(DELIMITER, rh_top_idx)) == string::npos)	return -2;
	// The "+ 2" is a ENDL.
	req_header = req.substr( rh_top_idx, rh_end_idx - rh_top_idx + ENDL.length() );

	// Request body
	if( rh_end_idx + DELIMITER.length() != req.length() )
		req_set.body = req.substr( rh_end_idx + DELIMITER.length() );
	else
		req_set.body = "";

	//cerr << "Req header: " << req_header << endl;
	//cerr << "Req body: " << req_set.body << endl;

	return 0;
}


/**
	Parse the request line
*/
int HTTP::parseReqLine( const string &req_line )
{
#if WXDEBUG == 1
	cerr << "HTTP::separateReqLine()" << endl;
#endif

	/* Parse the method part */
	size_t met_end_idx;
	if( (met_end_idx = req_line.find(" ", 0)) == string::npos )	return -1;
	string method = req_line.substr(0, met_end_idx);
	req_set.idx = searchIndexFromToken(METHOD, METHOD_N, method);

	/* Parse (extract) the path part*/
	size_t path_top_idx = met_end_idx + 1, path_end_idx;
	if( (path_end_idx = req_line.find(" ", path_top_idx)) == string::npos )	return -1;
	req_set.path = req_line.substr(path_top_idx, path_end_idx - path_top_idx);

	//cerr << "Method: " << method << " (" << getReqMethod() << ")" << endl;
	//cerr << "Path: " << req_set.path << endl;

	return 0;
}


/**
	Parse the request header
*/
int HTTP::parseReqHeader( const string &req_header )
{
#if WXDEBUG == 1
	cerr << "HTTP::parseReqHeader()" << endl;
#endif

	for( int hdr_i = 0; hdr_i < HEADER_N; ++ hdr_i )
		req_set.header[hdr_i].clear();

	size_t colon_idx = 0, line_top_idx = 0;
	while((colon_idx = req_header.find(": ", line_top_idx)) != string::npos){
		string header = req_header.substr(line_top_idx, colon_idx - line_top_idx);
		int header_idx = searchIndexFromToken(HEADER, HEADER_N, header);

		size_t line_end_idx = req_header.find(ENDL, colon_idx);
		string header_str = req_header.substr( colon_idx + 2, line_end_idx - (colon_idx + 2) );
		req_set.header[header_idx] = header_str;
		line_top_idx = line_end_idx + 2;
	}

	//for( int i = 0; i < HEADER_N; i ++ )
	//	cerr << "\t" << HEADER[i].getHeader() << " = " << req_set.header[i] << endl;

	/* body  urlencoded ̏ꍇɂ̓fR[h */
	//cout << req_set.header[HDR_ID_CTYP] << endl;
	//cout << MIME::get(MIME::MIM_ID_URLENCODED) << endl;
	if( req_set.header[HDR_ID_CTYP] == MIME::get(MIME::MIM_ID_URLENCODED) ) {
		string body_decoded;
		char ch, hex;
		for( size_t i = 0; i < req_set.body.length(); ++ i ){
			ch = req_set.body.at( i );
			switch( ch ) {
			case '%':
				// 216i2
				hex = 0;
				for( int digit = 16; digit > 0; hex *= digit, digit /= 16 ) {
					ch = req_set.body.at( ++i );
					if( isdigit( ch ) ) hex += ch - '0';	 // 0`9
					else hex += toupper( ch ) - 'A' + 10; // A`F
					//printf("%02X\n", hex );
				}
				ch = hex;
				break;
			case '+':
				ch = ' ';
				break;
			default:
				break;
			}
			body_decoded.push_back( ch );
		}
		req_set.body = body_decoded;
		//cout << "[req body(decoded)]: " << req_set.body << endl;
	}

	return 0;
}


void HTTP::setHTMLTemplToResBody()
{
	int code = RESPONSE[res_set.idx].getStatusCode();
	const string &message = RESPONSE[res_set.idx].getStatusMessage();

	ostringstream oss;
	oss
		<< "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">"
		<< "<HTML>\r\n"
		<< "<HEAD><TITLE>" << code << " " << message << "</TITLE></HEAD>\r\n"
		<< "<BODY><H1>" << code << " " << message << "</H1></BODY>\r\n"
		<< "</HTML>\r\n";

	res_set.body = oss.str();

	const int content_type = MIME::MIM_ID_HTM;
	const size_t size = res_set.body.length();
	setResHdr( &content_type, &size );
}


/**
	Set the response headers.
	@	<I>	content_type
	@	<I>	content_length
*/
void HTTP::setResHdr( const int *content_type, const size_t *content_length )
{
#if WXDEBUG == 1
	cerr << "HTTP::setResHdr()" << endl;
#endif

	res_set.header[HDR_ID_CONN] = "close";

	if( content_type ) {
		res_set.header[HDR_ID_CTYP] = MIME::get(*content_type);
	}
	if( content_length ) {
		ostringstream oss;
		oss << *content_length;
		res_set.header[HDR_ID_CLEN] = oss.str();
	}

	setResHdrDateField();

#if WXDEBUG == 1
	cerr << "<<HTTP::setResHdr()" << endl;
#endif
}


/**
	Set the date field in the response header.
*/
void HTTP::setResHdrDateField()
{
#if WXDEBUG == 1
	cerr << "HTTP::setResHdrDateField()" << endl;
#endif

	static char *week[] = {
		"Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"
	};
	static char *month[] = { "",
		"Jan", "Feb", "Mar", "Apr", "May", "Jun",
		"Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
	};
	SYSTEMTIME systime;
	GetSystemTime( &systime );

	/* tH[}bgϊ */
	static char tmp[512];
	sprintf( tmp, "%s, %2d %s %4d %02d:%02d:%02d GMT",
		week[systime.wDayOfWeek], systime.wDay,
		month[systime.wMonth], systime.wYear,
		systime.wHour, systime.wMinute, systime.wSecond );

	res_set.header[HDR_ID_DATE] = tmp;
}


/**
	Search idx from an T_id_table.
	@ <I>	id_table				: the id table with the type of T_id_table
	@ <I> wxString &token
	@	return
*/
int HTTP::searchIndexFromToken( const IdStrTaple *taple, int taple_n, const string &token ) const
{
#if WXDEBUG == 1
	cerr << "HTTP::searchIndexFromToken()" << endl;
#endif

	int i;
	for( i = 0; i < taple_n - 1; i ++ ) {
		if( token.at(0) != taple[i].getStr().at(0) )	continue;
		if( token == taple[i].getStr() )			break;
	}

	return i;
}