#pragma once


#include <vector>
#include <string>
#include <cstring>
#include "mime.h"
#include "id-str-taple.h"


class HTTP
{
//=====================================
//	Public constants & types
//=====================================
public:
	static const std::string DELIMITER;

	enum method_id_t
	{
		MET_ID_GET = 0,				MET_ID_HEAD,
		MET_ID_PUT,						MET_ID_POST,
		MET_ID_UNKNOWN,
	};

//=====================================
//	Private constants & types
//=====================================
private: // Enumerator

	enum protocol_id_t
	{
		VER_ID_HTTP_10 = 0,		VER_ID_HTTP_11,
		VER_ID_UNKNOWN
	};

	enum response_id_t
	{
		RES_ID_OK =	0,		RES_ID_NO_CONTENT,
		RES_ID_BAD_REQ,		RES_ID_FORBIDDEN,
		RES_ID_NOT_FOUND,	RES_ID_METHOD_NA,
		RES_ID_INT_SRV_E,	RES_ID_NOT_IMPL,
		RES_ID_SRV_UNAV,	RES_ID_UNKNOWN
	};

	enum header_id_t
	{
		HDR_ID_ACCE = 0,	HDR_ID_HOST,
		HDR_ID_CONN,			HDR_ID_CLEN,
		HDR_ID_CTYP,			HDR_ID_DATE,
		HDR_ID_AENC,      HDR_ID_ALNG,
		HDR_ID_ACHR,      HDR_ID_USRA,
		HDR_ID_CCTR,      HDR_ID_RFRR,
		HDR_ID_ORIG,
		HDR_ID_UNKNOWN
	};

private: // The number of the ids
	static const int METHOD_N = MET_ID_UNKNOWN + 1;
	static const int VERSION_N = VER_ID_UNKNOWN + 1;
	static const int RESPONSE_N = RES_ID_UNKNOWN + 1;
	static const int HEADER_N = HDR_ID_UNKNOWN + 1;
	
private: // Id tables
	static const HTTPMethod METHOD[METHOD_N];
	static const HTTPVersion VERSION[VERSION_N];
	static const HTTPResponse RESPONSE[RESPONSE_N];
	static const HTTPHeader HEADER[HEADER_N];

private:	// constants
	static const std::string ENDL;

private:	// struct of the request and the response

	// HTTP request / response message
	struct msg_t
	{
		int idx;
		std::string header[HEADER_N];

		std::string path;	// Request only
		std::string body;
	};


//=====================================
//	Private members
//=====================================
private:
	msg_t req_set;
	msg_t res_set;
	
//=====================================
//	Public functions
//=====================================
public:
	HTTP() {}
	~HTTP() {}

public:
	int parseRequest( const std::string &req );

#if 0
	int processGet( const std::string &uri );
	int processPost( const std::string &body );
	int processUnknown();

	bool createResponse( std::string &buf );
#else

	int createResponse( std::string &buf );

#endif

public:	// Accessors for request
	int getReqMethodId()                const { return req_set.idx; }
	const std::string &getReqMethod()   const { return METHOD[req_set.idx].getName(); }
	const std::string &getReqPath()     const { return req_set.path; }
	int getReqContLen()                 const { return ::atoi( req_set.header[HDR_ID_CLEN].c_str() ); }
	const std::string &getReqContType() const { return req_set.header[HDR_ID_CTYP]; }
	const std::string &getReqMsgBody()  const { return req_set.body; }

public:	// Accessors for response
	int getResStatCode()                const { return RESPONSE[res_set.idx].getStatusCode(); }
	const std::string &getResStatMsg()  const { return RESPONSE[res_set.idx].getStatusMessage(); }
	//const std::string &getResContLen()  const { return res_set.header[HDR_ID_CLEN]; }
	int getResContLen()                 const { return ::atoi( res_set.header[HDR_ID_CLEN].c_str() ); }
	const std::string &getResContType() const { return res_set.header[HDR_ID_CTYP]; }
	const std::string &getResMsgBody()  const { return res_set.body; }

	int setResStatCode( int code );
	void setResContLen( size_t size );
	void setResContType( MIME::id_t id ) { res_set.header[HDR_ID_CTYP] = MIME::get( id ); }
	void setResMsgBody( const std::string &body ) { res_set.body = body; }

	void setResHdr( const int *content_type = NULL, const size_t *content_length = NULL );
	void setResHdr( int content_type, size_t content_size )
	{ setResHdr( &content_type, &content_size ); }
public:
	void set404()
	{
		setResStatCode( 404 );
		setResHdr( NULL, NULL );
	}

	void set200( const char* body = NULL )
	{
		setResStatCode( 200 );
		if( body ) {
			setResHdr( MIME::MIM_ID_HTML, strlen( body ) );
			setResMsgBody( body );
		}
		else
			setResHdr( MIME::MIM_ID_HTML, NULL );
	}

//=====================================
//	Private functions
//=====================================
private:
	int separateReq( const std::string &req, std::string &req_line, std::string &req_header );
	int parseReqLine( const std::string &req_line );
	int parseReqHeader( const std::string &req_header );

	void setHTMLTemplToResBody();

	//void setResHdr( const int *content_type = NULL, const size_t *content_length = NULL );
	void setResHdrDateField();	

	int searchIndexFromToken( const IdStrTaple *taple, int taple_n, const std::string &token ) const;
};
