/**********************************************************/
/* OpenCV Wrapper                                         */
/*  @ Author        : J. Sugiyama @ AISL.TUT              */
/*  @ Last update   : 24.Mar.2012                         */
/*--------------------------------------------------------*/
/*	Comment added                             01.Feb.2012 */
/*  OpenCV2.3.1 available                     24.Mar.2012 */
/**********************************************************/
#pragma once

#ifndef _USE_MATH_DEFINES
#define _USE_MATH_DEFINES
#endif
#include <cmath>

#include <cstdlib>

#include <opencv/cv.h>
#include <opencv/cxcore.h>


#if defined( CVMATH_LINK_LIBS_200 )
#	pragma comment( lib, "cv200.lib" )
#	pragma comment( lib, "cxcore200.lib" )
#elif defined( CVMATH_LINK_LIBS_210 )
#	ifdef _DEBUG
#		pragma comment( lib, "cv210d.lib" )
#		pragma comment( lib, "cxcore210d.lib" )
#	else
#		pragma comment( lib, "cv210.lib" )
#		pragma comment( lib, "cxcore210.lib" )
#	endif
#elif defined( CVMATH_LINK_LIBS_220 )
#	ifdef _DEBUG
#		pragma comment( lib, "opencv_core220d.lib" )
#	else
#		pragma comment( lib, "opencv_core220.lib" )
#	endif
#elif defined( CVMATH_LINK_LIBS_230 )
#	ifdef _DEBUG
#		pragma comment( lib, "opencv_core230d.lib" )
#	else
#		pragma comment( lib, "opencv_core230.lib" )
#	endif
#elif defined( CVMATH_LINK_LIBS_231 )
#	ifdef _DEBUG
#		pragma comment( lib, "opencv_core231d.lib" )
#	else
#		pragma comment( lib, "opencv_core231.lib" )
#	endif
#endif


//#include <iostream>
//using namespace std;
namespace opencv
{
namespace cvmath
{

	/*=======================================================
		radian  degree ɕϊ	
	=======================================================*/
	inline double rad2deg( double rad )
	{ return 180.0 * rad / M_PI; }


	/*=======================================================
		degree  radian ɕϊ	
	=======================================================*/
	inline double deg2rad( double deg )
	{ return M_PI * deg / 180.0; }


	/*=======================================================
		lower`upper̈l̐
	=======================================================*/
	inline double sampleUniform( double lower = 0.0, double upper = 1.0 )
	{ return lower + (upper - lower) * (double) rand() / (double) RAND_MAX; }


	/*=======================================================
		 m, W΍ s 1K̐
	=======================================================*/
	inline double sampleNormal1D( double m, double s )
	{
		double u;
		while( !(u = sampleUniform( 0, 1 )) );
		return m + s * sqrt( -2*log(u) ) * cos( sampleUniform(0, 2 * M_PI) );
	}

	/*=======================================================
		2WK̐	
	=======================================================*/
	inline void sampleNormal2D( double *x, double *y )
	{
		double u;
		while( !(u = sampleUniform( 0, 1 )) );
		double theta = sampleUniform( 0, 2*M_PI );

		double r = sqrt( -2*log(u) );

		if( x ) *x = r * cos( theta );
		if( y )	*y = r * sin( theta );
	}

} // namespace cvmath
} // namespace opencv

#include "cvmttype.h"
#include "cvmtvector.h"
#include "cvmtmatrix.h"
#include "cvmtq8n.h"
//#include "cvmteigen.h"
#include "cvmtutil.h"