/**********************************************************/
/* OpenCV Wrapper																					*/
/*	@ Author				:	J. Sugiyama @ AISL.TUT							*/
/*	@	Last update		:	01.Feb.2012													*/
/**********************************************************/
#pragma once


#include "cvmttype.h"
#include "cvmtvector.h"
#include <iostream>


namespace opencv
{
namespace cvmath
{
	
	/********************************************************
		sNX
	********************************************************/
	template < int CV_TYPE >
	class Matrix
	{
	public:
		CvMat *e;

	public:	// Constructors & destructor ====================
		/*=====================================================
			ftHgRXgN^
		=====================================================*/
		Matrix()
		{	e = NULL;	}

		
		/*=====================================================
			RXgN^ismہj
		=====================================================*/
		Matrix( int r, int c )
		{	e = cvCreateMat( r, c, CV_TYPE );	}


		/*=====================================================
			Rs[RXgN^
		=====================================================*/
		Matrix( const Matrix &src )
		{
			if( src.e ) {
				e = cvCreateMat( src.getRows(), src.getRows(), CV_TYPE );
				set( src );
			}
			else e = NULL;
		}


		/*=====================================================
			fXgN^
		=====================================================*/
		virtual ~Matrix()
		{
			cvReleaseMat( &e );
			e = NULL;
		}

	public:
		/*=====================================================
			sTCY̕ύX
		=====================================================*/
		void resize( int r, int c )
		{
			if( e != NULL )
				cvReleaseMat( &e );
			e = cvCreateMat( r, c, CV_TYPE );
		}


		/*=====================================================
			swb_̐
		=====================================================*/
		void createHeader( int r, int c )
		{
			if( e != NULL )
				cvReleaseMat( &e );
			e = cvCreateMatHeader( r, c, CV_TYPE );
		}

	public:
		/*=====================================================
			sTCYւ̃ANZX
		=====================================================*/
		int getRows() {	return e->rows; }
		int getCols() {	return e->cols; }

	public:
		typedef typename CVElem::CvElemTypeTraits< CV_TYPE >::ElemType elem_t;

		/*=====================================================
			vf(i,j) val ݒ
		=====================================================*/
		void set( int i, int j, double val )
		{	cvmSet( e, i, j, val );	}


		/*=====================================================
			vf(i,j)擾
		=====================================================*/
		elem_t get( int i, int j ) const
		{	return ( elem_t ) cvmGet( e, i, j );	}
		
	public:
		/*=====================================================
			doublez񂩂̃Rs[
		=====================================================*/
		void set( const double *M )
		{
			for( int r = 0; r < e->rows; ++r )
				for( int c = 0; c < e->cols; ++c )
					set( r, c, *(M ++) );
		}


		/*=====================================================
			doublez񂩂̃Rs[
		=====================================================*/
		void set( const Matrix &M )
		{	cvCopy( M.e, e );	}


	public:
		/*=====================================================
			vf(i,j)MRs[
		=====================================================*/
		void set( int i, int j, const Matrix &M )
		{
			for( int y = 0; y < M.e->rows; ++y )
				for( int x = 0; x < M.e->cols; ++x )
					set( i+y, j+x, M.get(y,x) );
		}


		/*=====================================================
			vf(i,j)M̓]uRs[
		=====================================================*/
		void setT( int i, int j, const Matrix &M )
		{
			for( int y = 0; y < M.e->rows; ++y )
				for( int x = 0; x < M.e->cols; ++x )
					set( i+x, j+y, M.get(y,x) );
		}

		
		/*=====================================================
			vf(i,j)ɗxNgvRs[
		=====================================================*/
		void set( int i, int j, const Vector< CV_TYPE > &v )
		{
			for( int y = 0; y < v.e->rows; ++y )
				set( i+y, j, v.get(y) );
		}


		/*=====================================================
			vf(i,j)ɗxNgv̓]u(=sxNg)Rs[
		=====================================================*/
		void setT( int i, int j, const Vector< CV_TYPE > &v )
		{
			for( int x = 0; x < v.e->rows; ++x )
				set( i, j+x, v.get(x) );
		}

	public:
		/*=====================================================
			Mɗvf(i,j)̕sRs[
		=====================================================*/
		void get( int i, int j, Matrix &M ) const
		{
			for( int y = 0; y < M.e->rows; ++y )
				for( int x = 0; x < M.e->cols; ++x )
					M.set( y, x, get(i+y, j+x) );
		}


		/*=====================================================
			Mɗvf(i,j)̓]us̕sRs[
		=====================================================*/
		void getT( int i, int j, Matrix &M ) const
		{
			for( int y = 0; y < M.e->rows; ++y )
				for( int x = 0; x < M.e->cols; ++x )
					M.set( y, x, get(i+x, j+y) );
		}


		/*=====================================================
			vɗvf(i,j)̗xNgRs[
		=====================================================*/
		void get( int i, int j, Vector< CV_TYPE > &v ) const
		{
			for( int vi = 0; vi < v.e->rows; ++vi )
				v.set( vi, get(i+vi, j) );
		}


		/*=====================================================
			vɗvf(i,j)̍sxNgRs[
		=====================================================*/
		void getT( int i, int j, Vector< CV_TYPE > &v ) const
		{
			for( int vi = 0; vi < v.e->rows; ++vi )
				v.set( vi, get(i, j+vi) );
		}

	public:
		/*=====================================================
			0NA
		=====================================================*/
		void setZero()
		{ cvZero( e ); }


		/*=====================================================
			Svfvalɐݒ
		=====================================================*/
		void setAll( double val )
		{	cvSet( e, cvScalarAll(val) );	}


		/*=====================================================
			Pʍs
		=====================================================*/
		void setI( double val = 1.0 )
		{	cvSetIdentity( e, cvScalarAll(val) );	}


		/*=====================================================
			]usMtɎ擾
		=====================================================*/
		void trans( Matrix &Mt ) const
		{	cvmTranspose( e, Mt.e );	}


		/*=====================================================
			]usMtɎ擾
		=====================================================*/
		void invert( Matrix &Minv ) const
		{	cvmInvert( e, Minv.e );	}

		
		/*=====================================================
			]usMtɎ擾
		=====================================================*/
		double trace() const
		{
			return cvmTrace( e );
		}

	public:
		/*=====================================================
			2x2s̋tsMinvɎ擾
		=====================================================*/
		void invert22( Matrix &Minv ) const
		{
			const double
				a = (*this)(0,0), b = (*this)(0,1),
				c = (*this)(1,0), d = (*this)(1, 1), ad_minus_bc = a*d - b*c;
			Minv(0,0) =  d / ad_minus_bc;	Minv(0,1) = -b / ad_minus_bc;
			Minv(1,0) = -c / ad_minus_bc;	Minv(1,1) =  a / ad_minus_bc;
		}


		/*=====================================================
			2x2s̍s񎮂̒l߂
		=====================================================*/
		typename CVElem::CvElemTypeTraits< CV_TYPE >::ElemType det22() const
		{
			return get(0,0) * get(0,1) - get(1,0) * get(0,1);
		}

	public: // s񉉎Z ======================================
		/*=====================================================
			s̃XJ[{	
		=====================================================*/
		void xS( double s )
		{
			for( int r = 0; r < e->rows; ++r )
				for( int c = 0; c < e->cols; ++c )
					set( r, c, get(r,c) * s );
		}


		/*=====================================================
			s̃XJ[{	
		=====================================================*/
		void SxM( double s, const Matrix &M )
		{
			cvCopy( M.e, e );
			xS( s );
		}


		/*=====================================================
			s alpha*A B + beta*C ̌vZ
		=====================================================*/
		void MxM( const Matrix &A, const Matrix &B, const double alpha = 1.0, const double beta = 0.0 )
		{ cvGEMM( A.e, B.e, alpha, e, beta, e ); }


		/*=====================================================
			s alpha*A^tB + beta*C ̌vZ
		=====================================================*/
		void MtxM( const Matrix &A, const Matrix &B, const double alpha = 1.0, const double beta = 0.0 )
		{	cvGEMM( A.e, B.e, alpha, e, beta, e, CV_GEMM_A_T );	}


		/*=====================================================
			s alpha*AB^t + beta*C ̌vZ	
		=====================================================*/
		void MxMt( const Matrix &A, const Matrix &B, const double alpha = 1.0, const double beta = 0.0 )
		{	cvGEMM( A.e, B.e, alpha, e, beta, e, CV_GEMM_B_T );	}


		/*=====================================================
			s alpha*A^tB^t + beta*C ̌vZ
		=====================================================*/
		void MtxMt( const Matrix &A, const Matrix &B, const double alpha = 1.0, const double beta = 0.0 )
		{	cvGEMM( A.e, B.e, alpha, e, beta, e, CV_GEMM_A_T + CV_GEMM_B_T );	}


		/*=====================================================
			s AtBA ̌vZ
		=====================================================*/
		void MtxMxM( const Matrix &A, const Matrix &B, Matrix &AtB )
		{
			AtB.MtxM( A, B );
			MxM( AtB, A );
		}


		/*=====================================================
			s ABAt ̌vZ
		=====================================================*/
		void MxMxMt( const Matrix &A, const Matrix &B, Matrix &AB )
		{
			AB.MxM( A, B );
			MxMt( AB, A );
		}


		/*=====================================================
			saA+B̌vZ
		=====================================================*/
		void MplusM( const Matrix &A, const Matrix &B )
		{	cvmAdd( A.e, B.e, e );	}


		/*=====================================================
			sA-B̌vZ
		=====================================================*/
		void MminusM( const Matrix &A, const Matrix &B )
		{	cvmSub( A.e, B.e, e );	}

	public: // Zq̃I[o[[h ==========================
		/*=====================================================
			vf(i,j)̎Q
		=====================================================*/
		elem_t &operator ()( int r, int c )
		{	return ((elem_t *) e->data.ptr)[r*e->cols + c];	}
		const elem_t &operator ()( int r, int c ) const
		{ return ((elem_t *) e->data.ptr)[r*e->cols + c]; }


		/*=====================================================
			LXg
		=====================================================*/
		operator CvMat *() { return e; }
		operator const CvMat *() const { return e; }

	public:
		void print() const
		{
			for( int i = 0; i < e->rows; ++i ) {
				std::cout << "|";
				for( int j = 0; j < e->cols; ++j )
					std::cout << " " << get(i,j) << " ";
				std::cout << "|" << std::endl;
			}
		}


	};

} // namespace cvmath
} // namespace opencv