/**********************************************************/
/* OpenCV Wrapper																					*/
/*	@ Author				:	J. Sugiyama @ AISL.TUT							*/
/*	@	Last update		:	01.Feb.2012													*/
/**********************************************************/
#pragma once


#include "cvmtmatrix.h"


namespace opencv
{
namespace cvmath
{
	template < int CV_ELEM_T >
	class RotationMatrixDecomposer
	{
	private:
		Matrix< CV_ELEM_T > Rx, Ry, Rz;
		Matrix< CV_ELEM_T > Ri, Rtmp;
		Vector< CV_ELEM_T > angles;
		const double thresh_mergin;
		typedef typename CVElem::CvElemTypeTraits< CV_ELEM_T >::ElemType elem_t;

	public:
		RotationMatrixDecomposer( double threshold_mergin = 0.00001 )
			: Rx( 3, 3 ), Ry( 3, 3 ), Rz( 3, 3 ), 
			Ri( 3, 3 ), Rtmp( 3, 3 ),
			angles( 3 ), thresh_mergin( threshold_mergin )
		{
			Rx.setI();	Ry.setI();	Rz.setI();
			angles.setZero();
		}

		~RotationMatrixDecomposer()
		{}

	public:
		const Vector< CV_ELEM_T > &
			decomposeRzRyRxEuler( const Matrix< CV_ELEM_T > &R )
		{
			double x, y, z;

			printf( "R\n" );
			R.print();

#			if 0
			if( (1 - thresh_mergin) < fabs(R(2,0)) 
				&& fabs(R(2,0)) < (1+thresh_mergin) )
			{ // R(2,0) = -sin(}90)
				y = -R(2,0)/fabs(R(2,0)) * 0.5*M_PI;
					// R(2,0) = -1  y = +90
					// R(2,0) = +1  y = -90
				x = atan2( -R(1,2), R(1,1) );
				z = 0;
			}
			else {
				x = atan2(  R(2,1), R(2,2) );
				y = atan2( -R(2,0)*(elem_t)sin(x), R(2,1) );
					// ̌vZł́Cxpx0Ȃ΁CƂɂ0ɂȂĂ܂
				z = atan2(  R(1,0), R(0,0) );
			}

			angles(0) = x;
			angles(1) = y;
			angles(2) = z;

			Rx(1,1) = cos(x);   Rx(1,2) = -sin(x);
			Rx(2,1) = -Rx(1,2); Rx(2,2) = Rx(1,1);

			Ry(0,0) = cos(y);   Ry(0,2) = sin(y);
			Ry(2,0) = -Ry(0,2); Ry(2,2) = Ry(0,0);

			Rz(0,0) =  cos(z);  Rz(0,1) = -sin(z);
			Rz(1,0) = -Rz(0,1); Rz(1,1) = Rz(0,0);

#			else
			/* zpxm肷 */
			// xNgR(1,0,0)^t(x,y)vf΂킩
			z = atan2( R(1,0), R(0,0) );
			
			/* z]̋tspāCRyRx̍]s߂ */
			Rz(0,0) =  cos(z);  Rz(0,1) = -sin(z);
			Rz(1,0) = -Rz(0,1); Rz(1,1) = Rz(0,0);
			Rz.trans( Ri );

			Rtmp.MxM( Ri, R );
				// R = RzRyRx
				//  RyRx = RziR
			
			/* y]߂ */
			// RyRx(=RziR)ɃxNg(1,0,0)^t|
			// xNg(x,z)vf΋܂
			y = atan2( -Rtmp(2,0), Rtmp(0,0) );

			/* y]̋tspāCRx̉]s߂ */
			Ry(0,0) = cos(y);   Ry(0,2) = sin(y);
			Ry(2,0) = -Ry(0,2); Ry(2,2) = Ry(0,0);
			Ry.trans( Ri );

			Rx.MxM( Ri, Rtmp );
			// Rx = RyiRyRx

			/* x] */
			x = atan2( Rx(2,1), Rx(1,1) );

			angles(0) = x;
			angles(1) = y;
			angles(2) = z;

#			endif

			printf("Rx\n");	Rx.print();
			printf("Ry\n");	Ry.print();
			printf("Rz\n");	Rz.print();

			printf("Ex: %.1f\nEy: %.1f\nEz: %.1f\n",
				rad2deg(x), rad2deg(y), rad2deg(z));

			return angles;
		}

	public:
		const Matrix< CV_ELEM_T > &getRx() const { return Rx; }
		const Matrix< CV_ELEM_T > &getRy() const { return Ry; }
		const Matrix< CV_ELEM_T > &getRz() const { return Rz; }
	};


	template < int CV_ELEM_T >
	class RotationManager
	{
	private:
		Vector< CV_ELEM_T > axis;
		Quaternion< CV_ELEM_T > q0, q1, q2, q01, q012;
		Matrix< CV_ELEM_T > R012;

	public:
		RotationManager()
			: axis(3), q0(), q1(), q2(), q01(), q012(), R012(3,3)
		{
			axis.setZero();
			q0.setI(), q1.setI(), q2.setI(), q01.setI(), q012.setI();
			R012.setI();
		}

		~RotationManager()
		{}
	
	public:
		const Matrix< CV_ELEM_T > &getRxyxEuler( double x0Eular_rad, double yEular_rad, double x1Eular_rad ) { return getR012Eular( 0, x0Eular_rad, 1, yEular_rad, 0, x1Eular_rad ); }
		const Matrix< CV_ELEM_T > &getRxyzEuler( double xEular_rad,  double yEular_rad, double zEular_rad  ) { return getR012Euler( 0, xEular_rad,  1, yEular_rad, 2, zEular_rad );  }
		const Matrix< CV_ELEM_T > &getRxzxEuler( double x0Eular_rad, double zEular_rad, double x1Eular_rad ) { return getR012Eular( 0, x0Eular_rad, 2, zEular_rad, 0, x1Eular_rad ); }
		const Matrix< CV_ELEM_T > &getRxzyEuler( double xEular_rad,  double zEular_rad, double yEular_rad  ) { return getR012Eular( 0, xEular_rad,  2, zEular_rad, 1, yEular_rad );  }
		const Matrix< CV_ELEM_T > &getRyxyEuler( double y0Eular_rad, double xEular_rad, double y1Eular_rad ) { return getR012Eular( 1, y0Eular_rad, 0, xEular_rad, 1, y1Eular_rad ); }
		const Matrix< CV_ELEM_T > &getRyxzEuler( double yEular_rad,  double xEular_rad, double zEular_rad  ) { return getR012Eular( 1, yEular_rad,  0, xEular_rad, 2, zEular_rad );  }
		const Matrix< CV_ELEM_T > &getRyzyEuler( double y0Eular_rad, double zEular_rad, double y1Eular_rad ) { return getR012Eular( 1, y0Eular_rad, 2, zEular_rad, 1, y1Eular_rad ); }
		const Matrix< CV_ELEM_T > &getRyzxEuler( double yEular_rad,  double zEular_rad, double xEular_rad  ) { return getR012Eular( 1, yEular_rad,  2, zEular_rad, 0, xEular_rad );  }
		const Matrix< CV_ELEM_T > &getRzxzEuler( double z0Eular_rad, double xEular_rad, double z1Eular_rad ) { return getR012Eular( 2, z0Eular_rad, 0, xEular_rad, 2, z1Eular_rad ); }
		const Matrix< CV_ELEM_T > &getRzxyEuler( double zEular_rad,  double xEular_rad, double yEular_rad  ) { return getR012Eular( 2, zEular_rad,  0, xEular_rad, 1, yEular_rad );  }
		const Matrix< CV_ELEM_T > &getRzyxEuler( double zEular_rad,  double yEular_rad, double xEular_rad  ) { return getR012Euler( 2, zEular_rad,  1, yEular_rad, 0, xEular_rad );  }
		const Matrix< CV_ELEM_T > &getRzyzEuler( double z0Eular_rad, double yEular_rad, double z1Eular_rad ) { return getR012Eular( 2, z0Eular_rad, 1, yEular_rad, 2, z1Eular_rad ); }
	private:
		const Matrix< CV_ELEM_T > &getR012Euler(
			int axis0, double Eular_axis0_rad,
			int axis1, double Eular_axis1_rad, 
			int axis2, double Eular_axis2_rad )
		{
			axis.setZero();
			axis(axis0) = Eular_axis0_rad;	q0.fromAxis( axis );	axis(axis0) = 0;
			axis(axis1) = Eular_axis1_rad;	q1.fromAxis( axis );	axis(axis1) = 0;
			axis(axis2) = Eular_axis2_rad;	q2.fromAxis( axis );	axis(axis2) = 0;
			q01.QxQ( q0, q1 );
			q012.QxQ( q01, q2 );
			q012.toR( R012 );
			return R012;
		}

	};
}
}