#include "cvmath.h"
#include <iostream>
#include "../../asm/intrinsics/sse.h"
#include <tbb/tick_count.h>


using namespace std;
using namespace opencv::cvmath;
using namespace intrinsics::sse;


void RPRt(float COS, float SIN, float4 *_P)
{
	const float4 CSCS = setR0toR3( +COS, +SIN, +COS, +SIN ), SCSC = setR0toR3( -SIN, +COS, -SIN, +COS );
	const float4 CCCC = setR0toR3( +COS, +COS, +COS, +COS ), SSSS = setR0toR3( +SIN, +SIN, +SIN, +SIN );
	float4 _0020, _0121, _0222, _0323, _1030, _1131, _1232, _1333;
	float4 AB_0, AB_1, AB_2, AB_3;

	const float4 *_P0 = _P+0, *_P2 = _P+2;
	_0020 = shufps(*_P0, *_P2, _MM_SHUFFLE(0,0,0,0));
	_0121 = shufps(*_P0, *_P2, _MM_SHUFFLE(1,1,1,1));
	_0222 = shufps(*_P0, *_P2, _MM_SHUFFLE(2,2,2,2));
	_0323 = shufps(*_P0, *_P2, _MM_SHUFFLE(3,3,3,3));
	++_P0; ++_P2;
	_1030 = shufps(*_P0, *_P2, _MM_SHUFFLE(0,0,0,0));
	_1131 = shufps(*_P0, *_P2, _MM_SHUFFLE(1,1,1,1));
	_1232 = shufps(*_P0, *_P2, _MM_SHUFFLE(2,2,2,2));
	_1333 = shufps(*_P0, *_P2, _MM_SHUFFLE(3,3,3,3));

	_0020 = mulps(CSCS, _0020);   _1030 = mulps(SCSC, _1030);
	_0121 = mulps(CSCS, _0121);		_1131 = mulps(SCSC, _1131);
	_0222 = mulps(CSCS, _0222);   _1232 = mulps(SCSC, _1232);
	_0323 = mulps(CSCS, _0323);		_1333 = mulps(SCSC, _1333);

	AB_0 = addps(_0020, _1030);
	AB_1 = addps(_0121, _1131);
	AB_2 = addps(_0222, _1232);
	AB_3 = addps(_0323, _1333);

	_0020 = mulps(CCCC, AB_0);	_1030 = mulps(SSSS, AB_1);
	_0121 = mulps(SSSS, AB_0);	_1131 = mulps(CCCC, AB_1);
	_0222 = mulps(CCCC, AB_2);	_1232 = mulps(SSSS, AB_3);
	_0323 = mulps(SSSS, AB_2);	_1333 = mulps(CCCC, AB_3);

	// RPRtAB_?͓]ů֌WɂȂĂ邪C
	// RPRt͑Ώ̍sȂ̂ŁĈ܂ܑOK
	-- _P;
	*(++_P) = subps(_0020, _1030);
	*(++_P) = addps(_0121, _1131);
	*(++_P) = subps(_0222, _1232);
	*(++_P) = addps(_0323, _1333);

}


int main(void)
{
#if 0
	const double COS = cos(1.0), SIN = sin(1.0);

	float4 _A[8];
	float4 _B[8];
	float4 _AB[8];
	float4 _ABAt[8];

	Matrix<CV_32F> A;			A.createHeader(4, 4),			A.e->data.fl = (float *) _A;
	Matrix<CV_32F> B;			B.createHeader(4, 4),			B.e->data.fl = (float *) _B;
	Matrix<CV_32F> AB;		AB.createHeader(4, 4),		AB.e->data.fl = (float *) _AB;
	Matrix<CV_32F> ABAt;	ABAt.createHeader(4, 4),	ABAt.e->data.fl = (float *) _ABAt;

	A.setZero();
	A(0,0) = A(2,2) = COS;	A(0,1) = A(2,3) = -SIN;
	A(1,0) = A(3,2) = SIN;	A(1,1) = A(3,3) =  COS;
	std::cout << "A "<< std::endl;		A.print();
	
	B(0,0) = 1;	B(0,1) = B(1,0) = 2;	B(0,2) = B(2,0) = 3;	B(0,3) = B(3,0) = 4;
							B(1,1) = 5;						B(1,2) = B(2,1) = 6;	B(1,3) = B(3,1) = 7;
																		B(2,2) = 8;						B(2,3) = B(3,2) = 9;
																													B(3,3) = 10;

	cout << "B "<< endl;		B.print();
	
	int LOOP = 1;
	tbb::tick_count t0, t1;

	t0 = tbb::tick_count::now();
	for(int i = 0; i < LOOP; ++ i)
	{
		AB.MxM(A, B);
		ABAt.MxMt(AB, A);
	}
	t1 = tbb::tick_count::now();
	cout << "AB "<< endl;			AB.print();
	cout << "ABAt "<< endl;		ABAt.print();
	cout << (t1 - t0).seconds() << endl;

	t0 = tbb::tick_count::now();
	for(int i = 0; i < LOOP; ++ i)
	{
		RPRt(COS, SIN, _B);
	}
	t1 = tbb::tick_count::now();
	cout << "ABAt" << endl;		B.print();
	cout << (t1 - t0).seconds() << endl;
#endif

#if 1
	Vector<CV_32F> v32f(3);	v32f(0) = 0;	v32f(1) = 1;	v32f(2) = 2;	v32f.print();
	Vector<CV_64F> v64f(3);	v64f(0) = 0;	v64f(1) = 1;	v64f(2) = 2;	v64f.print();
	
	Matrix<CV_32F> m32f(2, 3);	for(int i = 0; i < 2; ++ i) for(int j = 0; j < 3; ++ j)	m32f(i, j) = float(i*3+j);	m32f.print();
	Matrix<CV_64F> m64f(5, 3);	for(int i = 0; i < 5; ++ i) for(int j = 0; j < 3; ++ j)	m64f(i, j) = double(i*3+j);	m64f.print();
	
	Matrix<CV_32F> a(2, 2), b(2, 2), c(2, 2);	
	a.setI();
	b.setAll(10);
	c.MxM(a, b);

	c.print();
#endif

#if 0
	Matrix<CV_64F> Rx(3, 3);

	double rx[] = {
		1, 0, 0,
		0, cos(deg2rad(25)), sin(deg2rad(25)),
		0,-sin(deg2rad(25)), cos(deg2rad(25))
	};
	Rx.set(rx);

	Matrix<CV_64F> Ry(3, 3);
	double ry[] = {
		cos(deg2rad(90)), 0, sin(deg2rad(90)),
		0, 1, 0,
		-sin(deg2rad(90)), 0, cos(deg2rad(90))
	};
	Ry.set(ry);

	Matrix<CV_64F> R(3, 3);
	R.MxM(Rx, Ry);

	Matrix<CV_64F> Qx(3, 3);
	double qx[] = {
		1, 0, 0,
		0, cos(deg2rad(-25)), sin(deg2rad(-25)),
		0,-sin(deg2rad(-25)), cos(deg2rad(-25))
	};
	Qx.set(qx);

	Matrix<CV_64F> Qy(3, 3);
	double qy[] = {
		cos(deg2rad(-90)), 0, sin(deg2rad(-90)),
		0, 1, 0,
		-sin(deg2rad(-90)), 0, cos(deg2rad(-90))
	};
	Qy.set(qy);

	Matrix<CV_64F> Q(3, 3);
	Q.MxM(Qy, Qx);
	
	Matrix<CV_64F> S(3, 3);
	S.MxM(Q, R);
	std::cout << "S" << std::endl; S.print();

	Vector<CV_64F> p(3), q(3);
	p.set(0, 1);	p.set(1, 0);	p.set(2, 1);
	q.MxV(R, p);
	q.print();

	{		
		Quaternion<CV_64F> q, qi, qqi;
		q.set(0, 1);	q.set(1, 2);	q.set(2, 3);	q.set(3, 4);
		q.print();
		q.invert(qi);
		qqi.QxQ(q, qi);
		qqi.print();
	}


	Matrix<CV_64F> A(3, 3),	EVs(3, 3);
	Vector<CV_64F> evs(3);

	A.set(0, 0, 1.0);	A.set(0, 1, 0.0);	A.set(0, 2, 0.0);
	A.set(1, 0, 0.0);	A.set(1, 1, 1.0);	A.set(1, 2, 0.0);
	A.set(2, 0, 0.0);	A.set(2, 1, 0.0);	A.set(2, 2, 1.0);

	calculateEigenValueAndEigenVector33(A, evs, EVs);

	std::cout << "A" << std::endl;
	A.print();

	std::cout << "evs" << std::endl;
	evs.print();

	std::cout << "EVs" << std::endl;
	EVs.print();
#endif

	getchar();

	return 0;
}
