#include "state-machine.h"
#include <iostream>


using namespace state_machine;
using namespace std;


/**********************************************************
	StateMachine::ScenarioManager
===========================================================
		ftHgRXgN^
**********************************************************/
StateMachine::ScenarioManager::ScenarioManager()
{
	is_initialized = false;
}


/**********************************************************
		fXgN^
**********************************************************/
StateMachine::ScenarioManager::~ScenarioManager()
{
	finalize();
}


/**********************************************************
		(XML̃[h)
**********************************************************/
int StateMachine::ScenarioManager::initialize(
	const char *xml_path, vector< State > &state_tbl )
{
	if( is_initialized )
		return sys::returnByError< int >( -1,
			"StateMachine::ScenarioManager::initialize(): already initialized" );

	TiXmlDocument scenario( xml_path );
	
	if( !scenario.LoadFile() ) {
		string err_msg = "StateMachine::ScenarioManager::initialize(): ";
		err_msg += xml_path;
		return returnByNotFound< int >( -1, err_msg.c_str() );
	}

	//scenario.Print( stdout );
		
	int rc;
	if( rc = loadRTM( scenario ) )
		return rc;

	if( rc = loadStateMachine( scenario, state_tbl ) )
		return rc;

	is_initialized = true;

	return 0;
}


/**********************************************************
		I
**********************************************************/
int StateMachine::ScenarioManager::finalize()
{
	if( is_initialized ) {
		is_initialized = false;

		nameservers.clear();
		rtcs.clear();

	}
	return 0;
}


/**********************************************************
	<rtm>̃[h
**********************************************************/
int StateMachine::ScenarioManager::loadRTM( TiXmlDocument &scenario )
{
	cout << "SMScnrMgr::loadRTM()" << endl;
	// <rtm>
	TiXmlNode *rtm_node = NULL;

	if( !(rtm_node = scenario.FirstChild( "rtm" )) )
		return returnByNotFound< int >( -1, 
			"StateMachine::ScenarioManager::loadRTM(): Node <rtm>" );
	//cout << "Node:<" << rtm_node->Value() << ">" << endl;

	int rc;
	rc = loadNameServersInRTM( rtm_node );
	if( rc )	return rc;

	/* _ł͖
	rc = loadRTCsIInRTM( rtm_node );
	if( rc )	return rc;
	*/

	return 0;
}


/**********************************************************
	<nameservers>̃[h
**********************************************************/
int StateMachine::ScenarioManager::loadNameServersInRTM( TiXmlNode *rtm_node )
{
	cout << "SMScnrMgr::loadNameServersInRTM()" << endl;
	TiXmlNode *ns_node = NULL;
	if( !(ns_node = rtm_node->FirstChild( "nameservers" )) )
		return returnByNotFound< int >( -1, "Node <nameservers>" );
	//cout << "  Node:<" << ns_node->Value() << ">" << endl;

	ns_node = ns_node->FirstChild();
	vector< char > ns( strlen(ns_node->Value()) + 1, '\0' );
	strcpy( &ns[0], ns_node->Value() );
	for( const char *tok = strtok( &ns[0], ";" ); tok; tok = strtok( NULL, ";" ) )
		nameservers.push_back( tok );

	/*
	for( vector< string >::iterator it = nameservers.begin();
		it != nameservers.end();
		++ it )
		cout << "    " << it->c_str() << endl;
	*/

	return 0;
}


/**********************************************************
	<rtcs>̃[h
**********************************************************/
int StateMachine::ScenarioManager::loadRTCsIInRTM(
	TiXmlNode *rtm_node )
{
	cout << "SMScnrMgr::loadRTCsIInRTM()" << endl;

	TiXmlNode *rtcs_node = NULL;
	if( !(rtcs_node = rtm_node->FirstChild( "rtcs" )) )
		return returnByNotFound< int >( -1, "Node <rtcs>" );
	//cout << "  Node:<" << rtcs_node->Value() << ">" << endl;

	TiXmlNode *rtc_node = NULL;
	if( !(rtc_node = rtcs_node->FirstChild( "rtc" )) )
		return returnByNotFound< int >( -1, "Node <rtc>" );
	
	do {
		// ŏ͖̈OFirstChildŒTĂ̂ŁC
		// ł̎s͗L蓾Ȃ
		if( strcmp( rtc_node->Value(), "rtc" ) ) {
			string err_msg = "StateMachine::ScenarioManager::loadRTCsIInRTM(): ";
			err_msg += rtc_node->Value();
			return returnByUndefined< int >( -1, err_msg.c_str() );
		}

		TiXmlNode *node = NULL;
		if( !(node = rtc_node->FirstChild()) )
			return returnByNotFound< int >( -1,
				"StateMachine::ScenarioManager::loadRTCsIInRTM(): RTC name in Node <rtc>" );

		rtcs.push_back( node->Value() );
	} while( rtc_node = rtc_node->NextSibling( "rtc" ) );

	/*
	for( vector< string >::iterator it = rtcs.begin();
		it != rtcs.end();
		++ it )
		cout << "    " << it->c_str() << endl;
	*/

	return 0;
}


/**********************************************************
	<statemachine>̃[h
**********************************************************/
int StateMachine::ScenarioManager::loadStateMachine(
	TiXmlDocument &scenario, vector< State > &state_tbl )
{
	cout << "SMScnrMgr::loadStateMachine()" << endl;

	// <statemachine>
	TiXmlNode *state_machine_node = NULL;
	if( !(state_machine_node = scenario.FirstChild( "statemachine" )) )
		return returnByNotFound< int >( -1, 
			"StateMachine::ScenarioManager::loadStateMachine): Node <statemachine>" );

	int rc;
	rc = loadStatesInStateMachine( state_machine_node, state_tbl );
	if( rc )	return rc;

	return 0;
}


/**********************************************************
	<state>̃[h
**********************************************************/
int StateMachine::ScenarioManager::loadStatesInStateMachine(
	TiXmlNode *state_machine_node, vector< State > &state_tbl )
{
	cout << "SMScnrMgr::loadStatesInStateMachine()" << endl;

	TiXmlNode *state_node = NULL;
	int state_id_counter = 0;

	/* <state><name>ǂ݂ăe[u쐬 */
	if( !(state_node = state_machine_node->FirstChild( "state" )) )
		return returnByNotFound< int >( -1, 
		"StateMachine::ScenarioManager::loadStatesInStateMachine(): Node <state>" );

	do {
		State state;

		string name;
		if( loadValue( state_node, "name", name ) )
			return returnByNotFound< int >( -1,
			"StateMachine::ScenarioManager::loadStatesInStateMachine() :Node <name>" );

		state.initialize( state_id_counter ++, name.c_str() );
		state_tbl.push_back( state );
	} while( state_node = state_node->NextSibling( "state" ) );
		// ^OtNextSibling̓Rgs΂

	/* ڍדǂݍ */
	if( !(state_node = state_machine_node->FirstChild( "state" )) )
		return returnByNotFound< int >( -1, 
		"StateMachine::ScenarioManager::loadStatesInStateMachine(): Node <state>" );

	int i = 0;
	do {
		int rc;
		if( rc = loadStateInStateMachine( state_node, state_tbl, state_tbl.at(i) ) )
			return rc;

		i ++;
	} while( state_node = state_node->NextSibling( "state" ) );
	
	return 0;
}


/**********************************************************
	<state>̃[h(1ubN)
**********************************************************/
int StateMachine::ScenarioManager::loadStateInStateMachine(
	TiXmlNode *state_node, vector< State > &state_tbl, State &state )
{
	cout << ">> SMScnrMgr::loadStateInStateMachine()" << endl;

	TiXmlNode *node = NULL;

	// <name>
	string name;
	if( loadValue( state_node, "name", name ) )
		return returnByNotFound< int >( -1,
			"StateMachine::ScenarioManager::loadStateInStateMachine() :Node <name>" );
	// ɓǂݍ܂Ă閼OƔr
	if( state.getName() != name )
		return returnByNotFound< int >( -1,
			"StateMachine::ScenarioManager::loadStateInStateMachine() :Node <name>" );

	// <nextstate input="" timing="manual"> 1ȂȂ
	TiXmlNode *next_node = NULL;
	if( next_node = state_node->FirstChild() ){
		// 1畡邩Ȃ
		//  <next> mȂC<error> Ȃ
		do {
			if( !strcmp( "next", next_node->Value() ) ) {
				// <next>
				State::transition_t trans;
				int rc;
				if( rc = loadNextInState( next_node, state_tbl, trans ) )
					return rc;
				state.getTransitionTable().push_back( trans );
			}
			else if( !strcmp( "error", next_node->Value() ) ) {
				// <error>
				int rc;
				if( rc = loadErrorInState( next_node, state_tbl, state ) )
					return rc;
			}
			else if( !strcmp( "kill", next_node->Value() ) ) {
				int rc;
				if( rc = loadRTCKilledInState( next_node, state ) )
					return rc;
			}
			else if( !strcmp( "name", next_node->Value() ) ) {
				// ǂݍݍς
			}
			else {
				string err_msg = "StateMachine::ScenarioManager::loadStateInStateMachine() :Node <";
				err_msg += next_node->Value();
				err_msg += ">";
				returnByUndefined( -1, err_msg.c_str() );
			}

		} while( next_node = next_node->NextSibling() );
	}

	cout << "<< SMScnrMgr::loadStateInStateMachine()" << endl;

	return 0;
}


/**********************************************************
	<next>̃[h(1ubN)
**********************************************************/
int StateMachine::ScenarioManager::loadNextInState(
	TiXmlNode *next_node, vector< State > &state_tbl,
	State::transition_t &trans )
{
	cout << ">> SMScnrMgr::loadNextInState()" << endl;

	TiXmlElement *next_element = next_node->ToElement();
	TiXmlNode *node = NULL;

	// <next in> (Ȃꍇ͂ƂɂJ)
	string in = next_element ? 
		(next_element->Attribute( "in" ) ? next_element->Attribute( "in" ) : "")
		: "";
	// <next out> (Ȃꍇ͂ƂɂJ)
	string out = next_element ? 
		(next_element->Attribute( "out" ) ? next_element->Attribute( "out" ) : "")
		: "";

	// <nexts timing> (Ȃꍇ͎)
	bool manual = next_element ?
		( next_element->Attribute( "timing" ) ?
			(!strcmp( next_element->Attribute( "timing" ), "manual" ) ? true : false)
			: false )
		: false;

	// O (ȂƂȂ)
	string name;
	if( loadValue( next_node, name ) )
		return returnByNotFound< int >( -1,
			"StateMachine::ScenarioManager::loadNextInState(): name" );

	// ԖɈvԂe[uTo
	size_t idx;
	for( idx = 0; idx < state_tbl.size(); ++ idx ) {
		if( state_tbl.at(idx).getName() == name )
			break;
	}
	if( idx == state_tbl.size() ) {
		string err_msg = "StateMachine::ScenarioManager::loadNextInState(): State ";
		err_msg += name;
		return returnByNotFound< int >( -1, err_msg.c_str() );
	}

	trans.next = &state_tbl.at( idx );	// Ԃւ̃|C^
	trans.input = in;										// 
	trans.output = out;									// o
	trans.on_manual = manual;						// Jڎ̑ҋ@@

	cout << "<< SMScnrMgr::loadNextInState()" << endl;
	return 0;
}


/**********************************************************
**********************************************************/
int StateMachine::ScenarioManager::loadErrorInState(
	TiXmlNode *error_node, vector< State > &state_tbl,
	State &state )
{
	cout << ">> SMScnrMgr::loadErrorInState()" << endl;

	// O (ȂƂȂ)
	string name;
	if( loadValue( error_node, name ) )
		return returnByNotFound< int >( -1,
			"StateMachine::ScenarioManager::loadErrorInState(): name" );

	// G[ԖɈvԂe[uTo
	size_t idx;
	for( idx = 0; idx < state_tbl.size(); ++ idx ) {
		if( state_tbl.at(idx).getName() == name )
			break;
	}
	if( idx == state_tbl.size() ) {
		string err_msg = "StateMachine::ScenarioManager::loadErrorInState(): State ";
		err_msg += name;
		return returnByNotFound< int >( -1, err_msg.c_str() );
	}

	state.setError( &state_tbl.at( idx ) );	// Ԃւ̃|C^
	//cout << state.getError()->getName() << endl;
	//printf( "error = %p\n", state.getError() );

	cout << "<< SMScnrMgr::loadErrorInState()" << endl;
	return 0;
}

/**********************************************************
**********************************************************/
int StateMachine::ScenarioManager::loadRTCKilledInState(
	TiXmlNode *kill_node, State &state )
{
	cout << ">> SMScnrMgr::loadRTCKilledInState()" << endl;

	TiXmlNode *rtc_node = NULL;
	if( !(rtc_node = kill_node->FirstChild( "rtc" )) )
		return returnByNotFound< int >( -1, "Node <rtc>" );
	
	do {
		// ŏ͖̈OFirstChildŒTĂ̂ŁC
		// ł̎s͗L蓾Ȃ
		if( strcmp( rtc_node->Value(), "rtc" ) ) {
			string err_msg = "StateMachine::ScenarioManager::loadRTCKilledInState(): ";
			err_msg += rtc_node->Value();
			return returnByUndefined< int >( -1, err_msg.c_str() );
		}

		TiXmlNode *node = NULL;
		if( !(node = rtc_node->FirstChild()) )
			return returnByNotFound< int >( -1,
				"StateMachine::ScenarioManager::loadRTCKilledInState(): RTC name in Node <rtc>" );

		//cout << "kill:" << node->Value() << endl;
		state.getRTCsKilledOnError().push_back( node->Value() );
	} while( rtc_node = rtc_node->NextSibling( "rtc" ) );

	//getchar();

	cout << "<< SMScnrMgr::loadRTCKilledInState()" << endl;
	return 0;
}

/**********************************************************
	l̃[h
**********************************************************/
int StateMachine::ScenarioManager::loadValue(
	TiXmlNode *node, const char *node_name, string &value,
	const char *missing )
{
	cout << ">> SMScnrMgr::loadValue(0)" << endl;

	if( (node = node->FirstChild( node_name )) 
		&& (node = node->FirstChild()) )
	{
		// ǂ߂ꍇ
		value = node->Value();
		cout << "<< SMScnrMgr::loadValue(0-0)" << endl;
		return 0;
	}
	else if( missing ) {
		// ǂ߂ȂĂꍇ
		value = missing;
		cout << "<< SMScnrMgr::loadValue(0-1)" << endl;
		return 0;
	}

	// łȂꍇ
	cout << "<< SMScnrMgr::loadValue(0-*)" << endl;
	return -1;
}
int StateMachine::ScenarioManager::loadValue(
	TiXmlNode *node, string &value, const char *missing )
{
	cout << ">> SMScnrMgr::loadValue(1)" << endl;

	if( node = node->FirstChild() ) {
		// ǂ߂ꍇ
		value = node->Value();
		cout << "<< SMScnrMgr::loadValue(1-0)" << endl;
		return 0;
	}
	else if( missing ) {
		// ǂ߂ȂĂꍇ
		value = missing;
		cout << "<< SMScnrMgr::loadValue(1-1)" << endl;
		return 0;
	}

	// łȂꍇ
	cout << "<< SMScnrMgr::loadValue(1-*)" << endl;
	return -1;
}


/**********************************************************
	\
**********************************************************/
void StateMachine::ScenarioManager::print()
{
	cout << "Name servers: " << endl;
	for( vector< string >::iterator it = nameservers.begin();
		it != nameservers.end(); ++ it )
		cout << "  " << it->c_str() << endl;

	cout << "RTCs: " << endl;
	for( vector< string >::iterator it = rtcs.begin();
		it != rtcs.end(); ++ it )
		cout << "  " << it->c_str() << endl;

}
