/*********************************************************/
/* Critical section (WinAPI)                             */
/* @ Author: J.Sugiyama@AISL.TUT                         */
/* @ Date  : Mar.14.2012                                 */
/* @ Release note:                                       */
/*   Git test                                Mar.14.2012 */
/*********************************************************/
#pragma once


#ifndef WIN32_LEAN_AND_MEAN
#define WIN32_LEAN_AND_MEAN
#endif

#include <windows.h>


namespace sys
{


	//=============================================
	//
	//	Critical section class (NOT INHERITABLE)
	//
	//=============================================
	class CriticalSection
	{
	private:	// Field
		CRITICAL_SECTION section;

	public:	// Constructor & destructor
		/**
			Default constructor
		*/
		CriticalSection() {}


		/**
			Destructor
		*/
		~CriticalSection()
		{
			//std::cerr << ">> ~CriticalSection()" << std::endl;
			//std::cerr << "<< ~CriticalSection()" << std::endl;
		}

	private:
		// Forbidden
		CriticalSection(const CriticalSection &cs);
    CriticalSection &operator =(const CriticalSection &cs);

	public:	// Functions
		/**
			Create a handle of a critical section.
		*/
		void initialize() { InitializeCriticalSection(&section); }

		
		/**
			Finalize the handle of the cretical section.
		*/
		void finalize() { DeleteCriticalSection(&section); }


		/**
			Enter the critical section.
		*/
		void enter() { EnterCriticalSection(&section); }

		/**
			Enter the critical section.
		*/
		bool trytoEnter()
		{
			return TryEnterCriticalSection(&section) ? true : false;
		}


		/**
			Get out of the critical section.
		*/
		void exit() { LeaveCriticalSection(&section); }


	};


}